from recipe_scrapers.lecker import Lecker
from tests import ScraperTest


class TestLeckerScraper1(ScraperTest):

    scraper_class = Lecker
    test_file_name = "lecker_1"

    def test_host(self):
        self.assertEqual("lecker.de", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("lecker.de", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Gemüsepfanne mit Hähnchen, Zuckerschoten und Brokkoli",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual("Hauptgericht", self.harvester_class.category())

    def test_prep_time(self):
        self.assertEqual(0, self.harvester_class.prep_time())

    def test_cook_time(self):
        self.assertEqual(25, self.harvester_class.cook_time())

    def test_total_time(self):
        self.assertEqual(25, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://images.lecker.de/,id=7e976162,b=lecker,w=610,cg=c.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 Brokkoli",
                "150 g Zuckerschoten",
                "2 Lauchzwiebeln",
                "4 kleine Hähnchenbrustfilets (à ca. 140 g)",
                "2 EL Sonnenblumenöl",
                "4 EL Sojasoße",
                "50 ml Gemüsebrühe",
                "6 Stiele Koriander",
                "1 EL Sesamsaat",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Schritt 1\nBrokkoli putzen und in kleinen Röschen vom Strunk schneiden. Zuckerschoten putzen. Lauchzwiebeln putzen und in ca. 10 cm lange Stücke schneiden.\nSchritt 2\nBrokkoli, Zuckerschoten und Lauchzwiebeln in reichlich kochendem Salzwasser 2–3 Minuten garen. Herausnehmen und mit kaltem Wasser abspülen. Auf ein Sieb gießen und gut abtropfen lassen.\nSchritt 3\nFleisch trocken tupfen und in Würfel (ca. 1,5 x 1,5 cm) schneiden. Öl in einer weiten Pfanne oder einem Wok erhitzen. Fleisch darin, unter Wenden, ca. 3 Minuten kräftig anbraten. Hitze reduzieren und weitere ca. 2 Minuten braten. Gemüse, Sojasoße und Brühe zufügen und 3–4 Minuten dünsten. Dabei gelegentlich wenden.\nSchritt 4\nKoriander waschen und trocken schütteln und, bis auf einige Blätter zum Garnieren, samt Stiel grob hacken. Sesam und Koriander zur Gemüsepfanne geben und untermischen. Mit Salz und Pfeffer abschmecken und mit Koriander garnieren.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(3.9, self.harvester_class.ratings())

    def test_nutrients(self):
        self.assertEqual(
            {
                "calories": "260",
                "carbohydrateContent": "7",
                "fatContent": "7",
                "proteinContent": "38",
                "servingSize": "1 Portion",
            },
            self.harvester_class.nutrients(),
        )

    def test_cuisine(self):
        self.assertEqual(None, self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Unser beliebtes Rezept für Gemüsepfanne mit Hähnchen, Zuckerschoten und Brokkoli und mehr als 65.000 weitere kostenlose Rezepte auf LECKER.de.",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class Mindmegette(AbstractScraper):
    @classmethod
    def host(cls):
        return "www.mindmegette.hu"

    def title(self):
        return self.soup.find("h1", {"class": "title"}).get_text()

    def total_time(self):
        item_sibling = self.soup.find("span", {"class": "spriteTime"})
        time = item_sibling.find_next().get_text()

        return get_minutes(time)

    def image(self):
        image_relative_url = self.soup.find("img", {"itemprop": "image", "src": True})[
            "src"
        ]

        if image_relative_url is not None:
            image_relative_url = f"https://{self.host()}{image_relative_url}"

        return image_relative_url

    def ingredients(self):
        ingredients = []
        shopping_cart = self.soup.find("ul", {"class": "shopingCart"}).findAll("li")

        for ingredient in shopping_cart:
            amount_unit = (
                ingredient.find("span", {"class": "ingredient-measure"})
                .get("title")
                .strip()
            )
            ingredient_name = (
                ingredient.find("span", {"class": "ingredient-name"}).get_text().strip()
            )
            ingredients.append((amount_unit + " " + ingredient_name).strip())

        return ingredients

    def instructions(self):
        instructions = self.soup.find("div", {"class": "instructions"}).findAll("li")

        instructions_arr = []
        for instruction in instructions:
            for tag in instruction.findAll("h2"):
                tag.replaceWith("")
            instructions_arr.append(normalize_string(instruction.get_text()))

        return "\n".join(instructions_arr)

    def yields(self):
        item_sibling = self.soup.find("span", {"class": "spritePortion"})
        portion = item_sibling.find_next().get_text()

        return get_yields(portion)



# mypy: disallow_untyped_defs=False
import functools
import logging

from recipe_scrapers.settings import settings

from ._interface import PluginInterface

logging.basicConfig()
logger = logging.getLogger(__name__)


class OpenGraphImageFetchPlugin(PluginInterface):
    """
    If .image() method on whatever scraper return exception for some reason,
    do try to fetch the recipe image from the og:image on the page.

    Apply to .image() method on all scrapers if plugin is active.
    """

    run_on_hosts = ("*",)
    run_on_methods = ("image",)

    @classmethod
    def run(cls, decorated):
        @functools.wraps(decorated)
        def decorated_method_wrapper(self, *args, **kwargs):
            logger.setLevel(settings.LOG_LEVEL)
            class_name = self.__class__.__name__
            method_name = decorated.__name__
            logger.debug(
                f"Decorating: {class_name}.{method_name}() with OpenGraphImageFetchPlugin"
            )

            image = None
            try:
                image = decorated(self, *args, **kwargs)
            except Exception:
                pass

            if image:
                return image
            else:
                logger.info(
                    f"{class_name}.{method_name}() did not manage to find recipe image. OpenGraphImageFetchPlugin will attempt to do its magic."
                )
                image = self.soup.find(
                    "meta", {"property": "og:image", "content": True}
                )
                return image.get("content")

        return decorated_method_wrapper



# mypy: allow-untyped-defs

from recipe_scrapers.briceletbaklava import BricelEtBaklava
from tests import ScraperTest


class TestBricelEtBaklavaScraper(ScraperTest):

    scraper_class = BricelEtBaklava

    def test_host(self):
        self.assertEqual("briceletbaklava.ch", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Michel/Bricelet & Baklava", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Bürli de Saint-Gall (St. Galler Bürli)", self.harvester_class.title()
        )

    def test_category(self):
        self.assertEqual(
            "Boulangerie\nRecettes de pains", self.harvester_class.category()
        )

    def test_yields(self):
        self.assertEqual(
            "Recette pour 4 pains doubles 8 pièces de 140 g",
            self.harvester_class.yields(),
        )

    def test_image(self):
        self.assertEqual(
            "https://image.over-blog.com/D913ECH_mmWHTy8_qRECetDo6PU=/filters:no_upscale()/image%2F3215825%2F20210816%2Fob_e19375_pain-buerli-6374.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "150 g de farine bise (type 1100) *",
                "150 g d'eau légèrement tiède",
                "10 g de levain chef actif ou 1 g de levure fraîche",
                "300 g du levain de la veille",
                "470 g de farine mi-blanche (type 720) *",
                "325 g (+ ou – 1 cl) d’eau froide",
                "6 g de levure fraîche",
                "14 g de sel",
                "ICI",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Mettre le levain chef dans un bol,\nVerser l’eau tiédie (25°C) par dessus et à l’aide d’un fouet, bien mélanger l’ensemble.\nAjouter la farine\net mélanger pour en faire une pâte un peu collante.\nCouvrir et laisser le levain commencer sa fermentation pendant 3 heures à température ambiante. Déposer ensuite le bol au réfrigérateur (5 à 6°C) pendant au moins 12 heures jusqu’à 24 heures au maximum.\nSortir le levain du réfrigérateur 2 heures avant son emploi afin qu’il se remette en température.\nAjouter par dessus la levure fraîche\net ajouter encore l’eau froide\nafin de le décoller\net verser le tout dans la cuve du pétrin. Ajouter la farine et commencer le pétrissage en vitesse lente, ceci pour 10 bonnes minutes. Après 3 minutes de pétrissage, ajouter le sel. A la fin des 10 minutes, la pâte sera bien collante car très hydratée, c’est normal. Continuer le pétrissage maintenant en 2ème vitesse pendant encore 5 minutes, la pâte doit être très pétrie. Afin de contrôler son élasticité, étirer un morceau de pâte entre les doigts, elle devrait être aussi fine que du papier.\nAvec un peu d’huile, graisser légèrement un grand bol et y déposer la pâte.\nRecouvrir d’un torchon humide ou d’un couvercle et laisser lever entre 3 et 4 heures /selon la saison) à température ambiante. Durant ces 4 heures de pousse, procéder à plusieurs rabats, par exemple toutes les 30 minutes. Il s’agit de rabattre la pâte 3 à 4 fois sur elle-même. Une fois le pointage terminé, la pâte aura bien doublé de volume\nla verser sur le plan de travail fariné et découpez des pâtons de 140 g chacun.\nEnsuite, façonnez délicatement les pâtons en boule sans trop les serrer afin de ne pas les dégazer\nDéposer ces pâtons par deux, l'un à côté de l'autre en se touchant légèrement, sur un torchon bien fariné avec la soudure vers le haut.\nFaire un pli avec le torchon afin que la pâte ne s'étale pas trop et les recouvrir d’un deuxième torchon fariné puis d’un plastique et laisser lever entre 30 et 40 minutes, cette opération se nomme « L’apprêt ». Pendant ce temps, allumer le four sur 240°C en ayant soin d’y déposer une pierre de cuisson ou une plaque métallique retournée. L’apprêt terminé, retourner les pâtons, toujours accolés l’un l’autre et les déposer sur une planchette en bois bien farinée et d’un coup sec, les glisser sur la pierre ou la plaque brûlante, sans oublier de verser au fond du four un verre d’eau avant de fermer la porte. Après 15 minutes de cuisson, entrouvrir rapidement la porte du four afin d’en évacuer la vapeur puis, baisser la température à 220 et poursuivre la cuisson pendant encore 20 minutes. Une fois cuits, les bürli doivent être foncés et croustillant, les sortir et les laisser refroidir sur une grille.",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        self.assertEqual(
            "Le Bürli de Saint-Gall est un petit pain en pâte levée, en forme de ballon. Juste avant l’enfournement ils sont accolés par deux et même quelquefois par 4 avec une cuisson très intense car les saint-gallois les apprécient bien cuits et croustillants. On les associe spontanément à la célèbre saucisse de veau du même canton, car lors de fêtes populaires, à la OLMA notamment, ces deux produits sont indissociables. On tient la saucisse grillée d’une main et le Bürli de l’autre et on les déguste alternativement, sans moutarde. Ces pains sont confectionnés à partir de farine mi-blanche et bise, La pâte est très hydratée ce qui leurs confère une mie très alvéolée et goûteuse. Et juste encore quelques mots sur la saucisse de Saint-Gall. La première mention officielle de celle-ci remonte à 1438. Dans les statuts de la société des bouchers de l’époque, il était indiqué que cette saucisse comprenait une part de viande de veau, une de porc et de lard et une de lait frais. J’y ai travaillé dans ce canton, plus précisément dans le Toggenburg et je peux dire que la recette a très peu changé depuis son origine à part quelques petites variantes, comme le lait frais qui s’est transformé en poudre de lait par exemple. Cette saucisse est vraiment délicieuse et il s’en transforme des tonnes par là-bas et elle a même obtenu le label IGP en 2008. On en mangeait tous les jours, plus précisément lors du « znüni », en français : les « neuf heures ».",
            self.harvester_class.description(),
        )



# mypy: disallow_untyped_defs=False
import functools
import logging
from html.parser import HTMLParser
from io import StringIO

from recipe_scrapers.settings import settings

from ._interface import PluginInterface

logging.basicConfig()
logger = logging.getLogger(__name__)


# Taken from @jksimoniii 's PR:
# - https://github.com/hhursev/recipe-scrapers/pull/346
# Modified to use the new "Plugin" system
# Taken from https://stackoverflow.com/questions/753052/strip-html-from-strings-in-python
class MLStripper(HTMLParser):
    def __init__(self):
        super().__init__()
        self.reset()
        self.strict = False  # this setting appears to do nothing
        self.convert_charrefs = True
        self.text = StringIO()

    def handle_data(self, d):
        self.text.write(d)

    def get_data(self):
        return self.text.getvalue()


def strip_tags(html):
    s = MLStripper()
    s.feed(html)
    return s.get_data()


def stripper(string):
    # Deal with HTML and HTML Encoded Characters
    string = strip_tags(
        f"<tag>{string}<tag>"
    )  # This is a workaround, since HTMLParser expects valid markup
    string = strip_tags(
        f"<tag>{string}<tag>"
    )  # This is another workaround, handles "&amp;amp;"

    return string


class HTMLTagStripperPlugin(PluginInterface):
    """
    Run the output from the methods listed through the stripper function
    defined above.

    It is intended to strip away <html><tags></tags></html> seen inside the strings.
    We do not want em.
    """

    decorate_hosts = ("*",)
    run_on_methods = ("title", "instructions", "ingredients")

    @classmethod
    def run(cls, decorated):
        @functools.wraps(decorated)
        def decorated_method_wrapper(self, *args, **kwargs):
            logger.setLevel(settings.LOG_LEVEL)
            class_name = self.__class__.__name__
            method_name = decorated.__name__
            logger.debug(
                f"Decorating: {class_name}.{method_name}() with HTMLTagStripperPlugin plugin."
            )

            decorated_func_result = decorated(self, *args, **kwargs)

            if type(decorated_func_result) is list:
                return [stripper(item) for item in decorated_func_result]
            else:
                return stripper(decorated_func_result)

        return decorated_method_wrapper



from recipe_scrapers.ohsheglows import OhSheGlows
from tests import ScraperTest


class TestOhSheGlowsScraper(ScraperTest):

    scraper_class = OhSheGlows

    def test_host(self):
        self.assertEqual("ohsheglows.com", self.harvester_class.host())

    def test_image(self):
        self.assertEqual(
            "https://ohsheglows.com/gs_images/2019/06/Flourless-Peanut-Butter-Cookie-Ice-Cream-00950-3.jpg",
            self.harvester_class.image(),
        )

    def test_canonical_url(self):
        self.assertEqual(
            "https://ohsheglows.com/2019/06/29/obsession-worthy-peanut-butter-cookie-ice-cream/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Obsession-Worthy Peanut Butter Cookie Ice Cream",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Angela Liddon")

    def test_ratings(self):
        self.assertEqual(self.harvester_class.ratings(), 4.67)

    def test_total_time(self):
        self.assertEqual(22, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 batch Flourless Peanut Butter Cookies, divided",
                "2 (14-ounce/398 mL) cans full-fat coconut milk*",
                "1/2 cup (105 g) natural cane sugar",
                "3 tablespoons (45 mL) smooth natural peanut butter",
                "2 teaspoons (10 mL) pure vanilla extract",
                "1/4 + 1/8 teaspoon fine sea salt, or to taste",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Chill the ice cream bowl in the freezer overnight, or for at least 12 hours. This step is very important to ensure the ice cream thickens properly.\nPrepare the Flourless Peanut Butter Cookies. After baking, cool the cookies for 10 to 15 minutes, then transfer each one to a plate. Place in the freezer on a flat surface for a minimum of 25 minutes. As soon as you transfer the cookies to the freezer, get started on the ice cream.\nAdd the ice cream ingredients (entire cans of coconut milk, sugar, peanut butter, vanilla, and salt) to a blender and blend for about 8 to 10 seconds, until smooth (be sure not to blend longer than 10 seconds, as it may effect the final texture of your ice cream).\nPlace the frozen ice cream bowl into the ice cream maker, insert the churning arm, cover with the lid, and turn on the machine (if the instructions for your ice cream maker are different, please follow the directions that came with your machine). Slowly pour the mixture into the bowl as it churns. Churn for about 22 minutes, until the mixture has thickened into a very thin, soft-serve texture.\nOnce the cookies have been in the freezer for 25 minutes, chop 6 of the cookies into small, almond-sized chunks. Reserve the remaining 7 cookies, at room temperature, for later.\nAfter 22 minutes of churning, slowly add the chopped cookies, a handful at a time, to the mixture while the machine is still churning. I like to use a fork to gently push the chopped cookies into the ice cream and help it along. Churn another 5 to 8 minutes, until the ice cream has thickened a bit more. It will have a thick, soft-serve texture when ready. There will be some hardened ice cream along the inside of the bowl...I like to think of this as the chef’s extra helping (wink, wink)! Serve immediately, or for a firmer texture, transfer the ice cream to a loaf pan or airtight container and spread out smooth. At this stage, I like to crumble an extra cookie all over the top (and gently push it into the ice cream) to make it look extra-enticing, but this is optional. Cover and freeze for 2 hours for a more traditional ice cream firmness.\nTo serve, scoop into bowls or ice cream cones. Or, if you're feeling wild, make ice cream sandwiches with the leftover cookies...oh yea!!\nStorage tip: Leftovers can be stored in an airtight container in the freezer for 3 to 4 weeks. Be sure to cover the ice cream with a piece of wrap to prevent freezer burn. To soften, let the container rest on the counter for 20 to 30 minutes before scooping.",
            self.harvester_class.instructions(),
        )


# https://ohsheglows.com/2019/06/29/obsession-worthy-peanut-butter-cookie-ice-cream/



from recipe_scrapers.bowlofdelicious import BowlOfDelicious
from tests import ScraperTest


class TestBowlOfDeliciousScraper(ScraperTest):

    scraper_class = BowlOfDelicious

    def test_host(self):
        self.assertEqual("bowlofdelicious.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.bowlofdelicious.com/six-minute-seared-ahi-tuna-steaks/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Six-Minute Seared Ahi Tuna Steaks"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Elizabeth Lindemann")

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.bowlofdelicious.com/wp-content/uploads/2019/09/Ahi-Tuna-Steaks-square.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                '2 ahi tuna (yellowfin tuna) steaks ((about 4 oz. each, 1" thick - see notes for thinner or thicker))',
                "2 tablespoons soy sauce",
                "1 tablespoon toasted sesame oil (see notes)",
                "1 tablespoon honey (see notes)",
                "1/2 teaspoon kosher salt",
                "1/4 teaspoon black pepper (to taste)",
                "1/4 teaspoon cayenne pepper ((optional))",
                "1 tablespoon canola oil (or olive oil)",
                "green onions, toasted sesame seeds, and lime wedges (for serving (optional))",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Pat the ahi tuna steaks dry with a paper towel. Place on a plate or inside a plastic bag.\nMix the soy sauce (2 tablespoons), toasted sesame oil (1 tablespoon), honey (1 tablespoon) kosher salt (1/2 teaspoon- OMIT if marinating for more than a couple hours, see notes), pepper (1/4 teaspoon), and cayenne pepper (1/4 teaspoon) until honey is fully dissolved. Pour over the ahi tuna steaks and turn over to coat completely. Optional: allow to marinate for at least 10 minutes, or up to overnight in the refrigerator. Also optional: Reserve a spoonful or two of the marinade before coating the fish for drizzling on top after you've cooked it.\nHeat a medium skillet (preferably non-stick or a well-seasoned cast iron skillet) on medium-high to high until very hot ( or medium medium-high for nonstick). I recommend giving cast iron 3-5 minutes to get hot and nonstick about 1 minute, depending on how thick it is.\nAdd the canola oil (1 tablespoon) to the hot pan. Sear the tuna for 1 - 1½ minutes on each side for medium rare ( 2 -2½ minutes for medium-well to well, 30 seconds for very rare. See notes - this will vary based on thickness of the tuna steaks). (Note: different burners get hotter depending on your stove. Use your best judgement whether you use medium, medium-high, or high heat, as the marinade may burn if too high heat is used)\nRemove to a cutting board. Slice into 1/2 inch slices and serve garnished with green onions, toasted sesame seeds, and a squeeze of fresh lime juice, if desired.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.marmiton import Marmiton
from tests import ScraperTest


class TestMarmitonScraper(ScraperTest):

    scraper_class = Marmiton

    def test_host(self):
        self.assertEqual("marmiton.org", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.marmiton.org/recettes/recette_ratatouille_23223.aspx",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Ratatouille")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Mirabelle")

    def test_total_time(self):
        self.assertEqual(80, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "350 g d'aubergine",
                "350 g de courgette",
                "350 g de poivron de couleur rouge et vert",
                "350 g d'oignon",
                "500 g de tomate bien mûres",
                "3 gousses d'ail",
                "6 cuillères à soupe d'huile d'olive",
                "1 brin de thym",
                "1 feuille de laurier",
                "poivre",
                "sel",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Coupez les tomates pelées en quartiers,\n"
            "les aubergines et les courgettes en rondelles.\n"
            "Emincez les poivrons en lamelles\n"
            "et l'oignon en rouelles.\n"
            "Chauffez 2 cuillères à soupe d'huile dans une poêle\n"
            "et faites-y fondre les oignons et les poivrons.\n"
            "Lorsqu'ils sont tendres, ajoutez les tomates, l'ail haché, le thym et le laurier.\n"
            "Salez, poivrez et laissez mijoter doucement à couvert durant 45 minutes.\n"
            "Pendant ce temps, préparez les aubergines et les courgettes. "
            "Faites les cuire séparemment ou non dans l'huile d'olive pendant 15 minutes.\n"
            "Vérifiez la cuisson des légumes pour qu'ils ne soient plus fermes. "
            "Ajoutez les alors au mélange de tomates et prolongez la cuisson sur tout petit feu pendant 10 min.\n"
            "Salez et poivrez si besoin.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.8, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class FitMenCook(AbstractScraper):
    @classmethod
    def host(cls):
        return "fitmencook.com"

    def title(self):
        raw_title = self.soup.find("h2", {"class": "gap-none"}).get_text()
        title = raw_title.replace("\t", "")
        title = title.replace("\n", "")

        return title

    def total_time(self):
        return get_minutes(self.soup.find("span", {"class": "total-time"}))

    def yields(self):
        for h4 in self.soup.findAll("h4"):
            for strong in h4.findAll("strong"):
                raw_yield = strong.text
                for word in raw_yield.split():
                    if word.isdigit():
                        yields = word

        return get_yields("{} servings".format(yields))

    def ingredients(self):
        ingredients_parent = self.soup.find("div", {"class": "recipe-ingredients"})
        ingredients = ingredients_parent.findAll("li")

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        instructions_parent = self.soup.find("div", {"class": "recipe-steps"})
        instructions = instructions_parent.findAll("li")

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )



from recipe_scrapers.simplywhisked import SimplyWhisked
from tests import ScraperTest


class TestSimplyWhiskedScraper(ScraperTest):

    scraper_class = SimplyWhisked

    def test_host(self):
        self.assertEqual("simplywhisked.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.simplywhisked.com/buffalo-chicken-chili/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Buffalo Chicken Chili")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Melissa Belanger")

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 tablespoons olive oil",
                "1 small onion, chopped (about 1 cup)",
                "2 stalks celery, chopped",
                "6 garlic cloves, minced",
                "1 1/2 – 2 pounds ground chicken",
                "1 cup water (or reduced-sodium broth)",
                "1 15-ounce can petite diced tomatoes",
                "2 15-ounce cans chili beans, with sauce",
                "2 tablespoos chili powder",
                "2 teaspoons ground cumin",
                "2 teaspoons paprika",
                "1 bay leaf",
                "1 teaspoon salt",
                "1/2 teaspoon pepper",
                "1/2 cup buffalo wing sauce",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In a large stockpot or dutch oven *affiliate link, heat olive oil to medium-high. Add the bell pepper, onion, celery and garlic. Sauté until onions are translucent, about 5 minutes.\nAdd ground chicken. Breaking up the meat as chicken browns, cook until no longer pink, about 5 minutes.\nAdd water, tomatoes, beans, chili powder, cumin, bay leaf, buffalo sauce, and salt & pepper. Bring to a simmer.\nCover and allow chili to cook for at least 15 minutes, simmering to desired thickness.\nBefore serving, remove bay leaf and adjust seasoning with salt & pepper, to taste.",
            self.harvester_class.instructions(),
        )

    def test_image(self):
        return self.assertEqual(
            "https://www.simplywhisked.com/wp-content/uploads/2016/02/Buffalo-Chicken-Chili-4-1-225x225.jpg",
            self.harvester_class.image(),
        )



from recipe_scrapers.gimmesomeoven import GimmeSomeOven
from tests import ScraperTest


class TestGimmeSomeOvenScraper(ScraperTest):

    scraper_class = GimmeSomeOven

    def test_host(self):
        self.assertEqual("gimmesomeoven.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.gimmesomeoven.com/sangria/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Sangria")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Ali")

    def test_total_time(self):
        self.assertEqual(10, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("12 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.gimmesomeoven.com/wp-content/uploads/2019/06/Authentic-Spanish-Sangria-Recipe-3-1-225x225.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 bottles Spanish red wine (Rioja wine is most popular)",
                "1/2 cup brandy",
                "2 oranges, one juiced and one diced",
                "1 green apple, diced",
                "1 lemon, diced",
                "1 cinnamon stick",
                "optional sweetener: simple syrup* or maple syrup",
                "optional bubbles: lemon-lime soda, ginger ale or sparkling water",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Add the wine, brandy, orange juice, diced orange, diced apple, diced lemon and cinnamon stick to a large pitcher. Stir to combine. Taste and add in a few tablespoons of sweetener, if desired.\nCover and refrigerate for at least 30 minutes or up to 4 hours.\nServe the sangria over ice, topping off each glass with a splash of bubbly soda (or sparkling water) if desired.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



# mypy: allow-untyped-defs

import re

from ._abstract import AbstractScraper


class Smulweb(AbstractScraper):

    instruction_delimiter = re.compile(r"(\.|\))\s*([A-Z])")

    @classmethod
    def host(cls):
        return "smulweb.nl"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def category(self):
        return self.schema.category()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        schema_instructions = self.schema.instructions()
        return self.instruction_delimiter.sub(r"\1\n\2", schema_instructions)

    def ratings(self):
        return self.schema.ratings()

    def cuisine(self):
        return self.schema.cuisine()

    def description(self):
        return self.schema.description()



from recipe_scrapers.wikicookbook import WikiCookbook
from tests import ScraperTest


class TestWikiCookbookScraper(ScraperTest):

    scraper_class = WikiCookbook

    def test_host(self):
        self.assertEqual("en.wikibooks.org", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://en.wikibooks.org/wiki/Cookbook:Pumpkin_Pie",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Pumpkin Pie")

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "//upload.wikimedia.org/wikipedia/commons/thumb/1/14/Pumpkin_Pie.jpg/300px-Pumpkin_Pie.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 cups (480ml) milk, scalded",
                "2 cups (450g) pumpkin, cooked and strained (or plain canned pumpkin). Use the smaller 'Sugar Pumpkin' instead of the big 'Jack O Lantern' pumpkin. The smaller 'Sugar Pumpkin' has a firm and smooth texture while the larger 'Jack O Lantern' pumpkin has more stringy or fibrous texture and more watery for a flesh.",
                "1 cup (240ml) maple syrup",
                "1/8 cup (30g) sugar",
                "1 Tbsp. flour",
                "½ tsp. salt",
                "1 tsp. ginger",
                "1 tsp. cinnamon",
                "1/4 tsp. nutmeg (optional)",
                "2 large eggs, beaten",
                "1 unbaked nine-inch pie shell",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat oven to 350 °F (180 °C).\nBlend all ingredients, except the pie shell, together.\nPour into the unbaked pie shell.\nBake at 350 °F (180 °C) for 45 minutes.\nLet cool and serve.",
            self.harvester_class.instructions(),
        )

    def test_description(self):
        return self.assertEqual(
            "Pumpkin pie is a traditional American and Canadian holiday dessert. It consists of a pumpkin-based custard baked in a single pie shell. The pie is traditionally served with whipped cream.",
            self.harvester_class.description(),
        )



from recipe_scrapers.panelinha import Panelinha
from tests import ScraperTest


class TestPanelinhaScraper(ScraperTest):

    scraper_class = Panelinha
    test_file_name = "panelinha_1"

    def test_host(self):
        self.assertEqual("panelinha.com.br", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Rosbife")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Panelinha")

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 peça de filé mignon para rosbife (cerca de 750 g)",
                "1 colher (chá) de mostarda amarela em pó",
                "1 colher (chá) de páprica defumada",
                "azeite a gosto",
                "sal e pimenta-do-reino moída na hora a gosto",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preaqueça o forno a 220 ºC (temperatura alta). Retire a peça de filé mignon da geladeira e deixe em temperatura ambiente por 15 minutos, enquanto o forno aquece.\nNuma tigela pequena, misture a páprica com a mostarda em pó. Disponha a peça de filé mignon na tábua e tempere com sal, pimenta e a mistura de mostarda com páprica. Regue com ½ colher (sopa) de azeite e espalhe bem com as mãos por toda a superfície da carne.\nTransfira o filé mignon para uma assadeira grande e leve ao forno para assar por 15 minutos. Após esse tempo, diminua a temperatura para 180 ºC (temperatura média) e deixe o rosbife no forno por mais 10 minutos para assar a carne com o interior bem vermelhinho (mal passada). Se quiser ao ponto, deixe assar por mais 5 minutos.\nRetire a assadeira do forno e deixe o rosbife descansar por 10 minutos antes de cortar e servir – nesse período os sucos se redistribuem, deixando a carne mais suculenta.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.meljoulwan import Meljoulwan
from tests import ScraperTest


class TestMeljoulwanScraper(ScraperTest):

    scraper_class = Meljoulwan

    def test_host(self):
        self.assertEqual("meljoulwan.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Melissa Joulwan", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Chicken Pesto Meatballs in Marinara Sauce", self.harvester_class.title()
        )

    def test_category(self):
        self.assertEqual("Poultry,Recipes,Whole30", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(25, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://meljoulwan.com/wp-content/uploads/2014/03/pestochicken.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1/2 cup (packed) fresh basil leaves",
                "1/4 cup fresh parsley leaves",
                "1/4 cup walnuts",
                "1 clove garlic, smashed and peeled",
                "1 tablespoon extra-virgin olive oil",
                "3/4 teaspoon salt",
                "1/4 teaspoon ground black pepper",
                "dash crushed red pepper flakes",
                "1 pound ground chicken",
                "2 tablespoons warm water",
                "1/2 teaspoon cream of tartar",
                "1/4 teaspoon baking soda",
                "1 teaspoon extra-virgin olive oil",
                "1-2 cloves garlic, minced",
                "1 (14.5 ounce) can crushed tomatoes",
                "1 \xa0(8 ounce) can tomato sauce",
                "1/2 teaspoon salt",
                "1/4 teaspoon ground black pepper",
                "10 large fresh basil leaves, slivered",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "1. Place the basil, parsley, walnuts, garlic, olive oil, salt, pepper, and red pepper flakes in the bowl of a food processor and purée until it forms a uniform paste. You may need to scrape down the sides a few times to get it to the right texture.",
                    "2. Place the ground chicken in a large bowl, then add the pesto. In a small bowl or measuring cup, mix the water with the cream of tartar and baking soda \xa0— it will fizz a little. Add to the bowl with the chicken and mix until all ingredients are evenly distributed. Cover and refrigerate at least 20 minutes and up to overnight.",
                    "3. While the meat is chilling, start the sauce. Heat a large non-stick skillet over low heat and add the olive oil. When the oil is warm, about 2 minutes, add the crushed garlic and when it’s fragrant, about 30 seconds, add the tomatoes. Stir to combine and bring to a very low simmer.",
                    "4. Remove the chicken dough from the fridge. Moisten your hands with cold water and shake to remove excess. Measure a rounded tablespoon of chicken and roll into a ball between your palms. Place gently in the tomato sauce. Repeat until all the balls are nestled into the sauce. It’s OK if they touch each other and aren’t completely submerged.",
                    "5. Cook the meatballs in the tomato sauce, covered, about 25 minutes. Remove the lid from the pan and allow to cook another 5-10 minutes to allow the sauce to thicken a little. Add the fresh basil leaves, stir, and serve on spaghetti squash or zucchini noodles. Bonus if you drizzle with a little extra-virgin olive oil.",
                ]
            ),
            self.harvester_class.instructions(),
        )



from recipe_scrapers.arla import Arla
from tests import ScraperTest


class TestArlaScraper(ScraperTest):

    scraper_class = Arla

    def test_host(self):
        self.assertEqual("arla.se", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Arla Mat", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Reuben sandwich", self.harvester_class.title())

    def test_category(self):
        self.assertEqual(
            "Brunch, Förrätt, Huvudrätt, Mellanmål", self.harvester_class.category()
        )

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cdn-rdb.arla.com/Files/arla-se/235235459/f96b874a-5b9c-4936-b3e2-5071e76136c0.jpg?mode=crop&w=1300&h=525&ak=f525e733&hm=1de43e21",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "200 g kokt rimmad oxbringa, eller corned beef",
                "1 dl surkål",
                "1 dl riven Arla Präst® ost",
                "8 skivor surdegsbröd",
                "50 g Arla® Svenskt Smör, smält",
                "2 saltgurkor",
                "½ grön paprika",
                "1 schalottenlök",
                "1 dl majonnäs",
                "1 msk chilisås",
                "1 tsk paprikapulver",
                "1 tsk dijonsenap",
                "½ krm tabasco",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Dressing:\nBörja med dressingen. Finhacka paprika och lök. Blanda med övriga ingredienser och ställ åt sidan.\nSista instruktionen\nSkär köttet i tunna skivor. Låt surkålen rinna av ordentligt och blanda med osten.\nPensla bröden med smör på båda sidor. Lägg surkål och ost på hälften av bröden. Lägg på köttet. Klicka dressingen över köttet och lägg på resterande bröd.\nGrilla på båda sidor i en het grillpanna eller smörgåsgrill.\nDela grillspett på mitten och trä genom mackorna. Skär saltgurkorna på längden och fäst på grillspetten.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.5, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual("Amerikansk", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "En reuben sandwich är en amerikansk macka, vanligtvis med pastrami - men "
            "vi använder svensk oxbringa, saltgurka och så kallad rysk dressing.",
            self.harvester_class.description(),
        )



from recipe_scrapers.cookinglight import CookingLight
from tests import ScraperTest


class TestCookingLight(ScraperTest):

    scraper_class = CookingLight

    def test_host(self):
        self.assertEqual("cookinglight.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.cookinglight.com/recipes/avocado-black-bean-and-charred-tomato-bowl",
            self.harvester_class.canonical_url(),
        )

    def test_image(self):
        self.assertEqual(
            "https://imagesvc.meredithcorp.io/v3/mm/image?url=https%3A%2F%2Fimg1.cookinglight.timeinc.net%2Fsites%2Fdefault%2Ffiles%2Fstyles%2F4_3_horizontal_-_1200x900%2Fpublic%2F1542730855%2Favocado-black-bean-and-charred-tomato-bowl-detox-book-p28.jpg%3Fitok%3D7dflTpmM",
            self.harvester_class.image(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Avocado, Black Bean, and Charred Tomato Bowl"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Cooking Light")

    def test_total_time(self):
        self.assertEqual(10, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("1 serving", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            "\n".join(
                [
                    "1/2 cup Savory Stewed Black Beans, warmed",
                    "1 teaspoon olive oil",
                    "1/2 cup grape tomatoes",
                    "1/4 cup fresh corn kernels (from 1 ear)",
                    "1/2 medium-sized ripe avocado, thinly sliced",
                    "1 medium radish, very thinly sliced",
                    "2 tablespoons fresh cilantro leaves",
                    "1/4 teaspoon kosher salt",
                    "1/8 teaspoon black pepper",
                ]
            ),
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "\n".join(
                [
                    "Step 1 Place the black beans in a pile in the corner of a shallow bowl. Heat a small skillet over medium-high. Add the oil to the pan; swirl to coat. Add the tomatoes; cook until charred but not collapsing, about 3 minutes, shaking the pan once to turn the tomatoes. Place the tomatoes next to the beans in the bowl.",
                    "Step 2 Add the corn to the pan; cook until heated through, 2 to 3 minutes. Place the corn next to the tomatoes. Add the avocado slices, radish slices, and cilantro to the bowl. Sprinkle with the salt and pepper.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(None, self.harvester_class.ratings())



from recipe_scrapers.hellofresh import HelloFresh
from tests import ScraperTest


class TestHelloFreshScraper(ScraperTest):

    scraper_class = HelloFresh

    def test_host(self):
        self.assertEqual("hellofresh.com", self.harvester_class.host())

    def test_host_domain(self):
        self.assertEqual("hellofresh.co.uk", self.harvester_class.host(domain="co.uk"))

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.hellofresh.co.uk/recipes/thai-style-pork-stir-fry-wk-49-5a01c742450cfa39e02c5642",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            "Thai Style Pork Stir-Fry with Veggie Rice", self.harvester_class.title()
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "HelloFresh")

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "150 grams Basmati Rice",
                "1 unit(s) Echalion Shallot",
                "2 unit(s) Garlic Clove",
                "¼ unit(s) Red Chilli",
                "2 unit(s) Spring Onion",
                "150 grams Green Beans",
                "300 grams Pork Mince",
                "50 grams Ketjap Manis",
                "25 milliliter(s) Soy Sauce",
                "6 grams Fresh Thai Basil",
                "300 milliliter(s) Water for the Rice",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """a) Pour the boiling water into a large saucepan and bring back to the boil on high heat. b) When boiling, add the rice and cook for 8-10 mins, then drain in a sieve and set aside.\na) Meanwhile, trim the carrot then grate on the coarse side of your grater (no need to peel). b) Trim the spring onion then finely slice. Roughly chop the peanuts. Zest the lime then chop into wedges. c) Chop the pork into 2cm chunks. iIMPORTANT: Remember to wash your hands after handling raw meat.\na) In a small bowl, stir together the easy ginger, ketjap manis, soy sauce, honey and the juice of half the lime. Set aside.\na) Heat a splash of oil in a large frying pan on high heat. b) When hot, add the pork and stir-fry until browned all over, 4-5 mins.\na) Lower the heat to medium then pour the sauce into the pan. b) Cook, coating the pork in the sticky sauce, for 2-3 mins.iIMPORTANT: The pork is cooked when it is no longer pink in the middle. c) Meanwhile, in a large bowl gently toss together the rice, lime zest, carrot, half the spring onion and half the peanuts. Season to taste with salt and pepper if needed.\na) Serve the sticky pork on top of the veggie rice. b) Finish by pouring any sauce left in the pan over the top and scattering over the remaining peanuts and spring onion. c) Top with the remaining lime wedges. ENJOY!""",
            self.harvester_class.instructions(),
        )

    def test_nutrients(self):
        self.assertEqual(
            {
                "calories": "668 kcal",
                "fatContent": "20 g",
                "saturatedFatContent": "6 g",
                "carbohydrateContent": "83 g",
                "sugarContent": "20 g",
                "proteinContent": "38 g",
                "sodiumContent": "3.25 g",
                "servingSize": "360",
            },
            self.harvester_class.nutrients(),
        )

    def test_cuisine(self):
        self.assertEqual("Thai", self.harvester_class.cuisine())

    def test_category(self):
        self.assertEqual("main course", self.harvester_class.category())



import unittest

from recipe_scrapers import get_supported_urls, scraper_exists_for
from recipe_scrapers._utils import get_host_name


class TestMainMethods(unittest.TestCase):
    def test_get_supported_urls(self):
        urls = get_supported_urls()
        self.assertGreater(len(urls), 200)
        self.assertIn(get_host_name("https://www.hellofresh.nl/"), urls)
        self.assertIn(get_host_name("https://hellofresh.com/"), urls)

    def test_scraper_exists_for(self):
        self.assertFalse(scraper_exists_for("example.com"))
        self.assertTrue(scraper_exists_for("https://www.hellofresh.nl/"))
        self.assertTrue(
            scraper_exists_for("https://eatsmarter.de/rezepte/gruenkohl-kokos-suppe")
        )



# mypy: allow-untyped-defs

import extruct

from ._abstract import AbstractScraper
from ._utils import normalize_string


class ZeitWochenmarkt(AbstractScraper):
    def __init__(self, url, **kwargs):
        AbstractScraper.__init__(self, url, **kwargs)
        data = extruct.extract(
            self.soup.prettify(), syntaxes=["json-ld"], errors="log", uniform=True
        )
        for item in data["json-ld"]:
            if item.get("@type") == "ItemList":
                self.schema.data = item["itemListElement"][0]["item"]

    @classmethod
    def host(cls):
        return "zeit.de"

    def title(self):
        return self.schema.title()

    def author(self):
        return self.soup.find("a", {"rel": "author"}).get_text().strip()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        class_name = "recipe-list-collection__special-ingredient"
        special_ingredients = [
            normalize_string(item.text)
            for item in self.soup.find_all("p", {"class": class_name})
        ]
        return special_ingredients + self.schema.ingredients()

    def instructions(self):
        class_name = "article__subheading article__subheading--recipe article__item"
        subset = self.soup.find("h2", {"class": class_name})
        class_name = "paragraph article__item"
        return "\n".join(
            [
                normalize_string(item.text)
                for item in subset.find_all_next("p", {"class": class_name})
            ]
        )

    def ratings(self):
        return self.schema.ratings()



from recipe_scrapers.theclevercarrot import TheCleverCarrot
from tests import ScraperTest


class TestTheCleverCarrotScraper(ScraperTest):

    scraper_class = TheCleverCarrot

    def test_host(self):
        self.assertEqual("theclevercarrot.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.theclevercarrot.com/2017/12/how-to-make-sourdough-cinnamon-rolls-step-by-step-guide/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Soft Sourdough Cinnamon Rolls")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Emilie Raffa")

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.theclevercarrot.com/wp-content/uploads/2017/12/How-to-Make-Sourdough-Cinnamon-Rolls-a-step-by-step-guide-13-225x225.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "160 g (2/3 cup) milk, whole or 2%",
                "28 g (2 tbsp) unsalted butter, melted (see note below)",
                "1 large egg",
                "100 g (1/2 cup) bubbly, active sourdough starter",
                "24 g (2 tbsp) granulated sugar",
                "300 g (2½ cups) all-purpose flour (I use King Arthur)",
                "5 g (1 tsp) fine sea salt",
                "cooking spray or oil, for coating",
                "28 g (2 tbsp) unsalted butter",
                "100 g (1/2 cup) granulated sugar",
                "3 tsp. ground cinnamon",
                "1 level tbsp. flour",
                "2 tbsp unsalted butter, softened",
                "⅓ cup whipped cream cheese, room temperature",
                "¼- 1/2 cup powdered sugar, sifted (add more if you like it sweet!)",
                "1-2 tbsp milk",
                "For a richer dough, increase the butter to 115 (8 tbsp) and use 360 g (3 cups) flour total. A reader recommended this tip, and I have to say, it’s my preferred method.",
                "Make sure the melted butter and milk mixture has cooled slightly before making the dough. If it’s too hot, the dough will become incredibly sticky like cake batter (I’ve experienced this many times). If this happens to you, don’t worry- wait for the dough to cool down before adding more flour, if needed.",
                "Recent recipe update: flour has been added to the cinnamon-sugar filling as a binder to prevent the butter from leaking out of the rolls.",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Baker's Schedule\nOvernight Option\nMake the dough in the evening and let rise overnight. The following morning, roll, cut and shape the dough. Rest for 1-2 hours (second rise) before baking.\nAs an alternative, after resting for 1 hour, cover the dough and chill until ready to use. Rest at room temperature before baking. The dough should be plump and puffy before baking.\nMake-Ahead Option (Freeze): Place the cut & shaped cinnamon rolls into a parchment lined 9-inch springform pan. Cover with two layers of plastic wrap. Freeze until ready to use. The night before baking, remove the old plastic wrap and replace with fresh wrap (this prevents any condensation from dripping onto the rolls). Defrost overnight, about 10-12 hrs. at room temperature, approximately 67 F. Bake the following morning as directed.\nMake the Dough\nIn the evening: Combine the melted butter and milk in a small bowl. Cool slightly before using.\nAdd the egg, sourdough starter, and sugar to the bowl of a stand mixer fitted with the paddle attachment. Mix to combine. With the machine running, slowly pour in the milk mixture. Add the flour and salt. Continue mixing until a rough, sticky dough forms, about 1 minute. Scrape down the sides of the bowl. Cover with a damp towel and let rest for 30 minutes.\nAfter the dough has rested, switch to the dough hook. Knead on medium-low speed for 6-8 minutes (I use #2 or #3 on my stand mixer). The dough should feel soft, supple and pull away from the sides of the bowl when ready. If it’s too sticky add a small bit of flour.\nBulk Rise\nTransfer the dough to a medium-size bowl coated in butter. Cover with plastic wrap. Let rise overnight until double in size, about 8-12 + hrs. @ 67-68 F, depending on temperature.\nStretch and Fold the Dough (optional step): about 30 minutes- 1 hr. into the bulk rise stretch and fold the dough: grab a portion of the dough and stretch it upward. Fold it over toward the center of the bowl. Give the bowl a 1/4 turn; stretch and fold the dough again. Continue this technique until you’ve come full circle around the bowl (4 folds total). For video guidance, click here. This optional step will increase the overall volume of the rolls and aerate the dough.\nRoll the Dough\nIn the morning: Line a 9-inch springform pan with parchment paper. I like to scrunch the paper into a ball first, open it up, and then line the inside with enough excess to hang over the sides for easy removal. It tends to fit better this way.\nLightly oil and flour your countertop to prevent sticking. Coax the dough out of the bowl. Gently pat into a rough rectangle. Let rest for 10 minutes for easier rolling.\nDust the dough (and your rolling pin) with flour. Roll the dough into a 16 x 12-ish rectangle using a tape measure for accuracy. If the dough resists, let rest for 5-10 minutes and try again.\nMake the Cinnamon-Sugar Filling\nCombine the cinnamon, sugar and flour in a small bowl; set aside. Melt the 28 g (2 tbsp) of butter in a shallow pan or microwave. Once the butter has cooled slightly, brush the entire surface of the dough, including the top, bottom and sides. Sprinkle the dough with the cinnamon-sugar mixture leaving a 1/2-inch border around the edges. Smooth it out with your hands.\nShape & Cut the Dough\nStarting on the long side of the dough (16-inch), roll it into a log pressing down gently as you go. Take your time with this step. The log needs to be tight so the swirls stay in tact. You should end up seam side down. TIP: if the dough starts to get sticky from the heat of your hands, lightly oil or flour your fingertips, take a deep breath and try again.\nCut the dough into 2-inch sections using a oiled knife or bench scraper. I lightly “mark” the dough first to make sure each piece is roughly the same size.\nSecond Rise\nPlace the rolls into the lined pan and let rest for 1- 2 hours, or until the dough puffs up. Alternatively, if you’d like to chill or freeze the rolls, please refer to the “Make-Ahead” option in the Baker’s Schedule at the top of this recipe.\nBake the Cinnamon Rolls\nPreheat oven to 350 F. Bake the dough onto the center rack and bake for 35-40 minutes (check at the 30 minute mark). The tops should turn light golden brown when ready.\nRemove from the oven and cool in the pan for 15 minutes. This helps the butter to absorb back into the dough. Then lift up the rolls, while still on the parchment paper, and transfer to a wire rack.\nMake the Glaze\nWhile the rolls are baking or cooling make the glaze. Add softened butter, whipped cream cheese and sifted powdered sugar to the bowl of a stand mixer. Beat until smooth, thinning out the consistency with a little milk as needed. The ingredients must be soft and at room temperature for best results.\nTo serve, top the rolls with some of the glaze or lightly dust with powdered sugar. These rolls are best enjoyed slightly warm on the same day they are baked.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.delish import Delish
from tests import ScraperTest


class TestDelishRogueOlScraper(ScraperTest):

    scraper_class = Delish
    test_file_name = "delish_rogue_ol"

    def test_host(self):
        self.assertEqual(self.harvester_class.host(), "delish.com")

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.delish.com/cooking/recipe-ideas/a46303/baileys-cheesecake-recipe/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Baileys Cheesecake")

    def test_total_time(self):
        self.assertEqual(self.harvester_class.total_time(), 480)

    def test_yields(self):
        self.assertEqual(self.harvester_class.yields(), "10 servings")

    def test_ingredients(self):
        self.assertEqual(
            self.harvester_class.ingredients(),
            [
                "26 Oreo cookies",
                "4 tbsp. butter, melted, plus more for pan",
                "Pinch of kosher salt",
                "4 (8-oz.) bars cream cheese, softened",
                "1 1/2 c. granulated sugar",
                "1/4 c. cornstarch",
                "4 large eggs",
                "2/3 c. Baileys Irish Cream",
                "1 tsp. pure vanilla extract",
                "2/3 c. heavy cream",
                "2 c. semisweet chocolate chips",
            ],
        )

    def test_instructions(self):
        self.assertTrue(self.harvester_class.instructions())

        self.assertEqual(
            self.harvester_class.instructions(),
            'Preheat oven to 325º and butter an 8" or 9" springform pan. Make crust: In a food processor, blend Oreos with melted butter and salt. Pat into a springform pan and set aside while you make filling.\nMake cheesecake: In a large bowl using a hand mixer, beat cream cheese and sugar until completely smooth and fluffy. Add cornstarch, then add eggs. Add Baileys and vanilla.\nPour batter into crust and place on a large baking sheet.\nBake until center of cheesecake is only slightly jiggly, 1 hour 20 minutes to 1 hour 30 minutes. Let cheesecake cool in oven, 1 hour, then refrigerate until completely cool, 4 hours or up to overnight. (If you\'d like to use a water bath, tightly wrap outside of springform pan with two layers of aluminum foil. Transfer to a deep roasting pan and pour in enough boiling water to reach halfway up the cheesecake pan. Bake as directed.)\nWhen ready to serve, make ganache: In a small saucepan over low heat, heat heavy cream. Place chocolate in a heatproof bowl and pour hot cream on top. Let sit 3 minutes, then stir until creamy, until no lumps remain. Refrigerate ganache until slightly thick, 15 minutes, and spread all over cheesecake. Let set 10 minutes, then serve.',
        )



from recipe_scrapers.justataste import JustATaste
from tests import ScraperTest


class TestJustATasteScraper(ScraperTest):

    scraper_class = JustATaste

    def test_host(self):
        self.assertEqual("justataste.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Baked Chicken and Cheese Taquitos (Oven or Air Fryer)",
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Kelly Senyei")

    def test_yields(self):
        self.assertEqual("6 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.justataste.com/wp-content/uploads/2018/04/baked-air-fryer-cheese-taquitos.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 Tablespoons extra-virgin olive oil",
                "1 cup diced yellow onion",
                "2 cloves garlic, minced",
                "2 Tablespoons fresh lime juice",
                "1 1/2 teaspoons ground cumin",
                "1 1/2 teaspoons paprika",
                "1/4 teaspoon kosher salt",
                "1/4 teaspoon fresh black pepper",
                "3 cups shredded rotisserie chicken",
                "1 cup shredded cheddar or Mexican blend cheese",
                "12 (6-inch) flour tortillas",
            ],
            self.harvester_class.ingredients(),
        )

    def test_total_time(self):
        return self.assertEqual(55, self.harvester_class.total_time())

    def test_ratings(self):
        return self.assertEqual(4.5, self.harvester_class.ratings())

    def test_instructions(self):
        return self.assertEqual(
            "Preheat the oven to 425ºF and line a baking sheet with parchment paper. (See Kelly's Notes for air fryer instructions.)\nIn a large saucepan, heat the olive oil over medium heat. Add the diced onion and cook until it's translucent, 3 to 5 minutes. Add the garlic, and cook, stirring occasionally, for about 3 minutes until it's golden and fragrant.\nReduce the heat to low, and then add the lime juice, cumin, paprika, salt and black pepper to the pan, stirring to combine. Add the shredded chicken, tossing to combine.\nTransfer the chicken mixture to a large bowl and let it cool for 10 minutes, and then stir in the shredded cheese.\nArrange the tortillas on work surface then place about 3 tablespoons of the chicken mixture on the lower third of each tortilla. Tightly roll up the tortilla, secure it with a toothpick, and then place it seam-side down on the prepared baking sheet. Repeat the filling and rolling process with the remaining tortillas.\nBake the taquitos for 15 to 20 minutes until golden brown and crispy. (See below for air fryer instructions.) Serve with guacamole and salsa.\nKelly's Notes:\nTaquitos are traditionally made with corn tortillas, however you can use flour tortillas, which are much more pliable, and thus easier to roll.\nTo cook the taquitos in an air fryer: Preheat the air fryer to 400°F. Grease the basket with cooking spray then arrange a portion of the taquitos in an even layer. Coat the taquitos with cooking spray then air-fry them until golden brown and crispy, about 8 minutes.",
            self.harvester_class.instructions(),
        )



# mypy: disallow_untyped_defs=False
import re
from typing import Dict, Optional, Tuple, Union

from ._abstract import AbstractScraper
from ._schemaorg import SchemaOrg
from ._utils import normalize_string


class FOKSchema(SchemaOrg):
    def category(self):
        cat = super().category()
        r = re.compile('.*<a href=".+">(.+)</a>')
        m = r.match(cat)
        if m:
            return normalize_string(m[1])
        return cat


class ForksOverKnives(AbstractScraper):
    @classmethod
    def host(cls):
        return "forksoverknives.com"

    def __init__(
        self,
        url: Union[str, None],
        proxies: Optional[
            Dict[str, str]
        ] = None,  # allows us to specify optional proxy server
        timeout: Optional[
            Union[float, Tuple[float, float], Tuple[float, None]]
        ] = None,  # allows us to specify optional timeout for request
        wild_mode: Optional[bool] = False,
        html: Union[str, bytes, None] = None,
    ):
        super().__init__(
            url=url, proxies=proxies, timeout=timeout, wild_mode=wild_mode, html=html
        )
        self.schema = FOKSchema(self.page_data)

    def author(self):
        author = self.soup.find("div", attrs={"class": "post-info"}).find("a")
        return normalize_string(author.get_text())

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        yields = normalize_string(
            self.soup.find("i", attrs={"class": "icon-serving"}).next_sibling.get_text()
        )
        # Get the first string after "Makes".
        return yields.split(" ", 1)[1]

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        ratings = normalize_string(
            self.soup.find("div", attrs={"class": "headline"})
            .find("span", attrs={"class": "rated-count"})
            .get_text()
        )
        # Unwrap parens
        ratings = ratings[1:]
        # return the first element
        return float(ratings.split()[0])

    def category(self):
        return self.schema.category()



from recipe_scrapers.melskitchencafe import MelsKitchenCafe
from tests import ScraperTest


class TestMelsKitchenCafeScraper(ScraperTest):

    scraper_class = MelsKitchenCafe

    def test_host(self):
        self.assertEqual("melskitchencafe.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.melskitchencafe.com/licorice-caramels/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Licorice Caramels")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Mel")

    def test_yields(self):
        self.assertEqual("50 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.melskitchencafe.com/wp-content/uploads/2013/12/licorice-caramels3.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1/2 cup water",
                "2 cups sugar",
                "1 (14-ounce) can sweetened condensed milk",
                "1 cup light corn syrup",
                "3/4 cup salted butter",
                "2 teaspoons anise extract (see note)",
                "1/2 teaspoon black food coloring paste (optional; see note)",
                "1/4 teaspoon vanilla extract",
                "1/4 teaspoon salt",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Lightly butter an 8X8- or 9X9-inch pan and set aside.\nIn a heavy-bottomed 4-quart saucepan, combine the water, sugar, condensed milk, corn syrup, and butter. Bring the mixture to a boil over medium heat, stirring constantly with a heat-resistant rubber spatula. Clip a candy thermometer to the side of the pan, ensuring that the tip of the thermometer isn’t touching the bottom of the pan and is inserted at least 1-2 inches into the liquid (or according to your thermometer’s directions).\nContinue stirring gently while the mixture boils and cooks, until the caramels reach 242-244 degrees F. If the caramels seem to be scorching on the bottom of the pan, moderate the heat to a lower temperature. You can also test the caramels using a spoon and dropping a pea-sized amount of the hot caramel into cold water. If the cooled piece of caramel is firm but not hard, the caramel is properly cooked.\nRemove the pot from the heat and stir in the anise extract, food coloring, vanilla extract and salt. Pour the caramels into the prepared pan and allow to cool completely to room temperature, at least 2 hours.\nWhen cool, remove the sheet of caramels from the pan. Cut the caramels into pieces using a large knife or bench scraper. Wrap each caramel square in a bit of wax paper, twisting the ends to secure.",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.justbento import JustBento
from tests import ScraperTest


class TestJustBentoScraper(ScraperTest):

    scraper_class = JustBento

    def test_host(self):
        self.assertEqual("justbento.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://justbento.com/handbook/recipes-sides-and-fillers/bento-filler-orange-juice-carrots",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "Bento Filler: Orange Juice Carrots"
        )

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("1", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "http://justbento.com/files/bento/images/orangecarrots1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "The thick end of 3 large carrots",
                "Orange juice (freshly squeezed is best but from a carton or concentrate is fine)",
                "1/4 tsp. salt",
                "1 whole red chili pepper",
                "1 Tbs. soy sauce",
                "1/2 Tbs. maple syrup",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "Cut the top half of the carrots. Reserve the thin ends for another dish. Peel the parts you will use, and cut into fairly thick rounds. Cut out shapes at this point if you like.",
                    "Put the carrots and chili pepper in a pan, and add enough orange juice to cover. Add the salt. Bring up to a boil, and cook until the carrot slices crisp-tender, about 10 minutes. Add a little water if it looks like the pan will dry out before the carrots are cooked.",
                    "When the carrots are cooked, take out the whole chili pepper. Add the soy sauce and maple syrup, and rapidly boil until the juice is almost gone. Cool off before putting into your bento.",
                    "This will keep for about 3-4 days, well covered in the refrigerator.",
                ]
            ),
            self.harvester_class.instructions(),
        )



from recipe_scrapers.steamykitchen import SteamyKitchen
from tests import ScraperTest


class TestSteamyKitchenScraper(ScraperTest):

    scraper_class = SteamyKitchen

    def test_host(self):
        self.assertEqual("steamykitchen.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Rylee Foer", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Instant Pot Vietnamese Chicken Pho", self.harvester_class.title()
        )

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://steamykitchen.com/wp-content/uploads/2020/10/SteamyKitchen_2020_websmall.png",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 pounds bone-in chicken (either a whole chicken or bone-in parts: breast or thigh)",
                "1 tablespoon cooking oil",
                "2 teaspoons of whole coriander seeds (not ground coriander)",
                "2 star anise pods",
                '2" nub of ginger (peeled and sliced a few times)',
                "1/2 onion",
                "3 whole cloves garlic",
                "3 tablespoons fish sauce",
                "1 1/2 teaspoons sugar",
                "1 package dried rice noodles (about 10-12 ounces, prepared according to package instructions, and drained)",
                "1/2 onion (thinly sliced and soaked in cold water)",
                "1 handful fresh cilantro (chopped)",
                "few sprigs of Thai basil (or regular basil)",
                "1 lime (in wedges)",
                "1/2 pound fresh bean sprouts",
                "1 jalapeno chile (sliced)",
                "Sriracha and Hoisin Sauce",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            'Make the Broth\nTurn the pressure cooker to "sauté" and heat the oil until smoking. Add the onion, ginger slices and cook until nicely browned, about 4 minutes. Add the garlic cloves, coriander, and star anise and sauté for another 2 minutes until fragrant.\nAdd in the chicken and 2 quarts of water to cover the chicken and seal the pot. Set to pressure and cook 20 minutes on high.\nMake your rice noodles while your broth is cooking.\nWhen the cooking is complete, carefully release the pressure then remove the lid once all pressure is released. Remove the chicken from the pot and transfer to a large bowl and let chicken cool off a bit. Strain all the spices from the broth and discard. Turn the pressure cooker on "boil" or "sauté" to keep the broth very hot.\nSeason broth with the fish sauce and sugar. Taste. If the broth is too bland, adjust with more fish sauce and sugar.\nRemove the chicken meat from bones, shred with fingers. Set aside.\nMake the Pho Bowls\nDrain the onions, Set your table with the onions, all of the herbs and condiments so that each person can customize their own bowl.\nDivide the chicken and prepared noodles amongst the bowls.\nReturn the pho broth to a boil. Ladle the hot pho broth into each bowl, and serve.',
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(None, self.harvester_class.ratings())



from recipe_scrapers.jimcooksfoodgood import JimCooksFoodGood
from tests import ScraperTest


class TestJimCooksFoodGoodScraper(ScraperTest):

    scraper_class = JimCooksFoodGood

    def test_host(self):
        self.assertEqual("jimcooksfoodgood.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Jim", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("German Potato Salad", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("8 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://jimcooksfoodgood.com/wp-content/uploads/2019/09/1282A83C-D784-4DF4-9623-C580DC9B9840.jpeg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 Pounds Yukon Gold Potatoes ((Or a small white potato))",
                "2/3 Cup Sherry or Apple Cider Vinegar",
                "1/2 Cup dijon mustard",
                "1/2 Cup Bread and Butter Pickles",
                "1/4 Cup olive oil",
                "4 Tablespoons capers ((Plus a splash of brine))",
                "2 Teaspoons salt",
                "1/2 Cup Fresh Parsley",
                "1/4 Cup scallions",
                "1 Sprig Fresh Rosemary",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "Preheat oven to 400 deg F. Cut potatoes into rough bitesize chunks, it doesn't have to be exact. Arrange on two non-stick sprayed baking sheets, season with 1/2 of the salt, and roast for 30 minutes.",
                    "As potatoes cook, roughly chop the pickles, capers, rosemary, scallions, and parsley. Combine thoroughly with vinegar, mustard, oil, remaining teaspoon of salt, and some heavy grinds of black pepper.",
                    "When potatoes are cooked, immediately transfer to very large mixing bowl. Add the dressing slowly as you mix; the potatoes should gently break apart, which is a good thing. Serve warm or at room temperature.",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(5.0, self.harvester_class.ratings())



from recipe_scrapers.farmhousedelivery import FarmhouseDelivery
from tests import ScraperTest


class TestFarmhouseDeliveryScraper(ScraperTest):

    scraper_class = FarmhouseDelivery
    test_file_name = "farmhousedelivery_2"

    def test_host(self):
        self.assertEqual("recipes.farmhousedelivery.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "http://recipes.farmhousedelivery.com/one-pan-crispy-rosemary-chicken-thighs-roasted-radishes/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            "One Pan Crispy Rosemary Chicken Thighs + Roasted Radishes",
            self.harvester_class.title(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1/2 lemon, juiced",
                "1/4 c olive oil",
                "1 tsp spicy mustard",
                "1 head of garlic, minced",
                "2 tsp pink Himalayan salt",
                "1 tsp black pepper",
                "4 sprigs fresh rosemary, de-stemmed",
                "4 bone-in chicken thighs (Find them here.)",
                "3 red potatoes, diced",
                "1 bunch radishes, quartered",
                "1 red onion, sliced",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "\n".join(
                [
                    "❶ Preheat your oven to 400°F.",
                    "❷ In a medium sized mixing bowl, whisk together olive oil, lemon juice, mustard, garlic, salt , pepper and rosemary.",
                    "❸ Layer potatoes, radishes and onions on a baking sheet. Lightly drizzle with the tinest bit of olive oil. The chicken thighs will drip fat for cooking, so we don’t want to overdo it with the oil.",
                    "❹ Pat chicken thighs dry and rub with seasoning mixture. Be sure to get in under the skin too.",
                    "❺ Place chicken thighs over your vegetable layer.",
                    "❻ Roast for 55 minutes, then broil for 3 minutes for an extra crisp factor! Enjoy!",
                    "– For more lifestyle tips and recipes from Rebecca, you can find her on Instagram at @xxrlilly",
                ]
            ),
            self.harvester_class.instructions(),
        )

    def test_image(self):
        self.assertEqual(
            "http://recipesfhd.wpengine.com/wp-content/uploads/2019/05/FHD-Chic-Thigh-Recipe-225x300.jpeg",
            self.harvester_class.image(),
        )



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._exceptions import SchemaOrgException
from ._utils import normalize_string


class Lecker(AbstractScraper):
    @classmethod
    def host(cls):
        return "lecker.de"

    def author(self):
        return self.schema.author()

    def title(self):
        try:
            return self.schema.title()
        except TypeError:
            return (
                self.soup.find(
                    "header", {"class": "article-header article-header--article"}
                )
                .find("h1")
                .get_text()
            )

    def category(self):
        return self.schema.category()

    def prep_time(self):
        return self.schema.prep_time()

    def cook_time(self):
        return self.schema.cook_time()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        return self.schema.ingredients()

    def instructions(self):
        if self.schema.instructions():
            return self.schema.instructions()
        else:
            divs = self.soup.find_all("div", {"class": "js-quizToggle"})
            for d in divs:
                if d.find("span", "article__shifted-jump-label"):
                    return normalize_string(d.get_text())

    def ratings(self):
        return self.schema.ratings()

    def nutrients(self):
        return self.schema.nutrients()

    def cuisine(self):
        try:
            return self.schema.cuisine()
        except SchemaOrgException:
            return None

    def description(self):
        return self.schema.description()



from recipe_scrapers.twopeasandtheirpod import TwoPeasAndTheirPod
from tests import ScraperTest


class TestTwoPeasAndTheirPodScraper(ScraperTest):

    scraper_class = TwoPeasAndTheirPod

    def test_host(self):
        self.assertEqual("twopeasandtheirpod.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.twopeasandtheirpod.com/baked-chicken-taquitos/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Chicken Taquitos")

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Maria Lichty")

    def test_total_time(self):
        self.assertEqual(40, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("20 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 cups shredded chicken, (we use rotisserie chicken)",
                "1/2 teaspoon ground cumin",
                "1/2 teaspoon ground chili powder",
                "1/2 teaspoon kosher salt",
                "1/4 teaspoon garlic powder",
                "1/4 teaspoon paprika",
                "2 teaspoons fresh lime juice",
                "1 cup shredded cheddar or Mexican blend cheese",
                "20 corn tortillas",
                "Shredded lettuce",
                "Diced tomatoes",
                "Guacamole",
                "Sour Cream",
                "Chopped Green Onion",
                "Crumbled Queso Fresco",
                "Pico de Gallo",
                "Salsa",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat the oven to 425 degrees F. Spray a large baking sheet with nonstick cooking spray and set aside.\nIn a medium bowl, combine the shredded chicken with the cumin, chili powder, salt, garlic powder, paprika, and fresh lime juice. Stir until chicken is well coated with the seasonings. Stir in the shredded cheese.\nGet two paper towels damp and place two tortillas at a time in between the paper towels. Place in the microwave for 20-30 seconds. Remove from the microwave and roll up the taquitos.\nPlace a heaping tablespoon of the chicken and cheese mixture in the center of the tortilla and roll it up tightly. Place the taquito, seam side down on the prepared baking sheet. Continue rolling taquitos until the tortillas and filling are gone. You should have about 20 taquitos.\nSpray the taquitos generously with nonstick cooking spray. Bake for 15-20 minutes or until taquitos are golden brown and crispy. Remove from the oven and serve warm with desired toppings.",
            self.harvester_class.instructions(),
        )

    def test_image(self):
        return self.assertEqual(
            "https://www.twopeasandtheirpod.com/wp-content/uploads/2017/03/Baked-Chicken-Taquitos-1.jpg",
            self.harvester_class.image(),
        )



import unittest

from recipe_scrapers import (
    NoSchemaFoundInWildMode,
    WebsiteNotImplementedError,
    scrape_me,
)


class TestExceptions(unittest.TestCase):
    def test_website_not_implemented_error(self):
        with self.assertRaises(WebsiteNotImplementedError):
            scrape_me("https://example.com/recipe")

    def test_no_schema_found_in_wild_mode_error(self):
        exception = NoSchemaFoundInWildMode("example.com")

        self.assertEqual(exception.url, "example.com")
        self.assertEqual(exception.message, "No Recipe Schema found at example.com.")



# mypy: allow-untyped-defs

from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class Yummly(AbstractScraper):
    @classmethod
    def host(cls):
        return "yummly.com"

    def title(self):
        found = self.soup.find("h1")
        return found.get_text() if found else None

    def author(self):
        return self.soup.find("a", {"class": "markdown-link"}).get_text()

    def total_time(self):
        data = self.soup.findAll("div", {"class": "recipe-summary-item"}, limit=2)
        return get_minutes(data[1]) if data else None

    def yields(self):
        return get_yields(self.soup.find("div", {"class": "servings"}))

    def ingredients(self):
        ingredients = self.soup.findAll("li", {"class": "IngredientLine"})

        return (
            [
                " ".join(
                    normalize_string(span.get_text())
                    for span in ingredient.select(
                        """
                        span[class^=amount],
                        span[class^=unit],
                        span[class^=ingredient],
                    span[class^=remainder]"""
                    )
                )
                for ingredient in ingredients
            ]
            if ingredients
            else None
        )

    def instructions(self):
        instructions = self.soup.findAll("li", attrs={"class": "prep-step"})
        return (
            "\n".join(normalize_string(instr.get_text()) for instr in instructions)
            if instructions
            else None
        )



from recipe_scrapers.sundpaabudget import SundPaaBudget
from tests import ScraperTest


class TestSundPaaBudgetScraper(ScraperTest):

    scraper_class = SundPaaBudget

    def test_host(self):
        self.assertEqual("sundpaabudget.dk", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            "One pot pasta med kyllingekebab",
            self.harvester_class.title(),
        )

    def test_category(self):
        self.assertEqual(
            "Kylling,One pot",
            self.harvester_class.category(),
        )

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_cook_time(self):
        self.assertEqual(20, self.harvester_class.cook_time())

    def test_prep_time(self):
        self.assertEqual(10, self.harvester_class.prep_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://sundpaabudget.dk/wp-content/uploads/2021/08/20210803123311_IMG_0714.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "300 g kyllingekebab",
                "1 spsk olie",
                "2 alm. løg",
                "2 tomater",
                "2 peberfrugter",
                "300 g fuldkornsspaghetti (el. pasta)",
                "125 g flødeost m. hvidløg (el. chili)",
                "7 dl vand",
                "1 bouillonterning",
                "0,5 tsk spidskommen",
                "0,5 tsk paprika (evt. røget)",
                "chili eller cayennepeber efter smag",
                "salt og peber",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Rengør og hak løg, peberfrugt og tomater i tern.
Opvarm olie i en stor gryde og svits kyllingekebaben et par minutter. Tilsæt grøntsagerne og lad det svitse med yderligere 2 minutter.
Tilsæt spaghetti, flødeost, vand, bouillonterning og krydderier.
Lad retten simre ved middelvarme til spaghettien er aldente og vandet er kogt næsten ind til en cremet sauce.
Smag til med salt og peber.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(3.9, self.harvester_class.ratings())

    def test_author(self):
        self.assertEqual("Britt // Sund på budget", self.harvester_class.author())

    def test_description(self):
        self.assertEqual(
            "Nem og lækker one pot med kyllingekebab (du kan også bruge okse/lam, hvis du hellere vil det).",
            self.harvester_class.description(),
        )

    def test_language(self):
        self.assertEqual("da-DK", self.harvester_class.language())

    def test_site_name(self):
        self.assertEqual("Sund på budget", self.harvester_class.site_name())



from recipe_scrapers.thepioneerwoman import ThePioneerWoman
from tests import ScraperTest


class TestThePioneerWomanScraper(ScraperTest):

    scraper_class = ThePioneerWoman

    def test_host(self):
        self.assertEqual("thepioneerwoman.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.thepioneerwoman.com/food-cooking/recipes/a10720/patty-melts/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Patty Melts")

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://hips.hearstapps.com/hmg-prod.s3.amazonaws.com/images/patty-melt-1597698088.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 stick butter",
                "1 whole large onion, halved and sliced",
                "1 1/2 lb. ground beef",
                "salt and pepper, to taste",
                "5 dashes Worcestershire sauce",
                "8 slices Swiss cheese",
                "8 slices rye bread",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "In a medium skillet, melt 2 tablespoons of butter over medium-low heat. Throw in the sliced onions and cook slowly for 20 to 25 minutes, stirring occasionally, until the onions are golden brown and soft.\nIn a medium bowl, mix together the ground beef, salt & pepper, and Worcestershire. Form into 4 patties.\nMelt 2 tablespoons butter in a separate skillet over medium heat. Cook the patties on both sides until totally done in the middle.\nAssemble patty melts this way: Slice of bread, slice of cheese, hamburger patty, 1/4 of the cooked onions, another slice of cheese, and another slice of bread. On a clean griddle or in a skillet, melt 2 tablespoons butter and grill the sandwiches over medium heat until golden brown. Remove the sandwiches and add the remaining 2 tablespoons of butter to the skillet. Turn the sandwiches to the skillet, flipping them to the other side. Cook until golden brown and crisp, and until cheese is melted.\nSlice in half and serve immediately!",
            self.harvester_class.instructions(),
        )

    def test_category(self):
        return self.assertEqual("main dish,meat", self.harvester_class.category())

    def test_cook_time(self):
        return self.assertEqual(25, self.harvester_class.cook_time())

    def test_prep_time(self):
        return self.assertEqual(10, self.harvester_class.prep_time())

    def test_ratings(self):
        return self.assertEqual(5.0, self.harvester_class.ratings())

    def test_author(self):
        return self.assertEqual("Ree Drummond", self.harvester_class.author())

    def test_cuisine(self):
        return self.assertEqual("", self.harvester_class.cuisine())

    def test_description(self):
        return self.assertEqual(
            "Who doesn’t love a good old-fashioned patty melt? Here's the easiest way to make one at home that's beyond delicious.",
            self.harvester_class.description(),
        )



from recipe_scrapers.kitchenstories import KitchenStories
from tests import ScraperTest


class TestKitchenStoriesScraper(ScraperTest):

    scraper_class = KitchenStories

    def test_host(self):
        self.assertEqual("kitchenstories.com", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            "Special strawberry jam with matcha", self.harvester_class.title()
        )

    def test_category(self):
        self.assertEqual("Jams & Marmalades", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(80, self.harvester_class.total_time())

    def test_cook_time(self):
        self.assertEqual(0, self.harvester_class.cook_time())

    def test_prep_time(self):
        self.assertEqual(20, self.harvester_class.prep_time())

    def test_image(self):
        self.assertEqual(
            "https://images.kitchenstories.io/wagtailOriginalImages/R2630-photo-final-1.jpg",
            self.harvester_class.image(),
        )

    def test_nutrients(self):
        self.assertEqual(
            {
                "calories": "359 cal",
                "carbohydrateContent": "91 g",
                "fatContent": "1 g",
                "proteinContent": "2 g",
                "servingSize": "1 Servings",
            },
            self.harvester_class.nutrients(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1.75 lbs strawberries",
                "1.5 cups jam sugar",
                "1 tsp matcha powder",
                "0.33 cup water",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Place a small plate in the freezer. Wash the strawberries and cut them into quarters. In a large pot, combine the jam sugar and strawberries, stirring until well combined. Let that sit for approx. 1 hr.\nIn the meantime, mix the matcha powder with half of the water in a small bowl to make a paste. Gradually stir in the remaining water until everything is well mixed and dissolved. Set aside.\nAfter the strawberries have rested and there's some juice in the pot, set the pot over medium heat and bring to a simmer. Cook for approx. 5 min., then add the matcha to the pot and continue cooking for approx. 1 min.\nTo test whether the jam will set up once it's cooled, place a small spoonful of the hot jam on the plate from the freezer and hold the plate at an angle. If the mass becomes solid after a short time and you can swipe your finger through the jam leaving an unconnected gap, the test is passed and the jam is ready; if not, continue cooking and repeating the test. Pour the still hot jam into sterilized preserving jars, seal, and let cool. Enjoy!",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(1.0, self.harvester_class.ratings())

    def test_author(self):
        self.assertEqual("Christian Ruß", self.harvester_class.author())

    def test_cuisine(self):
        self.assertEqual("International", self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "This recipe gives homemade strawberry jam an exciting update: Matcha!",
            self.harvester_class.description(),
        )

    def test_language(self):
        self.assertEqual("en", self.harvester_class.language())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, normalize_string


class TudoGostoso(AbstractScraper):
    @classmethod
    def host(cls):
        return "tudogostoso.com.br"

    def title(self):
        return normalize_string(self.soup.find("h1").get_text())

    def total_time(self):
        return get_minutes(self.soup.find("time", {"class": "dt-duration"}))

    def ingredients(self):
        ingredients_html = self.soup.findAll("span", {"class": "p-ingredient"})

        return [
            normalize_string(ingredient.get_text()) for ingredient in ingredients_html
        ]

    def instructions(self):
        instructions_html = self.soup.findAll(
            "div", {"class": "instructions e-instructions"}
        )

        return "\n".join(
            normalize_string(instruction.get_text())
            for instruction in instructions_html
        )



from recipe_scrapers.thewoksoflife import Thewoksoflife
from tests import ScraperTest


class TestThewoksoflifeScraper(ScraperTest):

    scraper_class = Thewoksoflife

    def test_host(self):
        self.assertEqual("thewoksoflife.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://thewoksoflife.com/whole-wheat-mantou/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(), "The Perfect Whole Wheat Mantou Recipe"
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "Judy")

    def test_yields(self):
        self.assertEqual("12 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://thewoksoflife.com/wp-content/uploads/2018/01/whole-wheat-mantou-9-1.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 \u2154 cups warm milk ((400 ml))",
                "1 teaspoon active dry yeast ((3 grams))",
                "1 tablespoon sugar ((12 grams))",
                "2 \u00be cups all-purpose flour ((400 grams))",
                "1\u00bc to 1\u00bd cups whole wheat flour ((about 170-200 grams; how much you\u2019ll need is dependent on the humidity in your kitchen))",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertTrue(
            self.harvester_class.instructions().startswith(
                "Heat the milk until warm to the touch (not hot). Then "
            )
        )
        self.assertEqual(len(self.harvester_class.instructions()), 1786)



import unittest
from typing import Any, Iterator, Optional, Tuple

import responses


class ScraperTest(unittest.TestCase):

    maxDiff = None
    test_file_name: Optional[str] = None
    test_file_extension = "testhtml"
    scraper_class: Any

    @classmethod
    def expected_requests(cls) -> Iterator[Tuple[str, str, str]]:
        """
        Descriptions of the expected requests that the scraper-under-test will make, as
        tuples of: HTTP method, URL, path-to-content-file
        """
        filename = cls.test_file_name or cls.scraper_class.__name__.lower()
        path = f"tests/test_data/{filename}.{cls.test_file_extension}"
        yield responses.GET, "https://test.example.com", path

    @classmethod
    def setUpClass(cls):
        with responses.RequestsMock() as rsps:
            start_url = None
            for method, url, path in cls.expected_requests():
                start_url = start_url or url
                with open(path, encoding="utf-8") as f:
                    rsps.add(method, url, body=f.read())

            cls.harvester_class = cls.scraper_class(url=start_url)



from recipe_scrapers.rainbowplantlife import RainbowPlantLife
from tests import ScraperTest


class TestRainbowPlantLifeScraper(ScraperTest):

    scraper_class = RainbowPlantLife

    def test_host(self):
        self.assertEqual("rainbowplantlife.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Nisha Vora", self.harvester_class.author())

    def test_title(self):
        self.assertEqual(
            "Vegan Brown Butter Peach Cobbler", self.harvester_class.title()
        )

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("10 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://rainbowplantlife.com/wp-content/uploads/2020/11/peachblueberrrycobbler286of1029.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 pound (454g) ripe but relatively firm peaches",
                "1/2 cup (75-85g) fresh blueberries",
                "1/3 cup (45g) brown sugar or coconut sugar",
                "1 teaspoon ground cinnamon",
                "1/4 teaspoon nutmeg ((I recommend freshly grated nutmeg))",
                "Scant 1/2 teaspoon ground ginger",
                "2 pinches of ground cardamom ((optional))",
                "10 tablespoons (140g) vegan butter*",
                "1 1/2 cups (180g) all-purpose flour",
                "Scant 2/3 cup (60g) (old-fashioned rolled oats)",
                "1 cup (200g) organic cane sugar",
                "1/4 - 1/2 teaspoon kosher salt",
                "2 teaspoons baking powder",
                "1 1/3 cups (320 mL) full-fat oat milk***",
                "1 1/2 teaspoons pure vanilla extract",
                "1/4 teaspoon pure almond extract ((optional))",
                "For serving: vegan vanilla ice cream ((optional))",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Arrange a rack in the middle of your oven and preheat it to 375°F/190°C.\nCut the peaches in half and remove the pits, then cut the peaches into 1/4-1/2 inch thick slices (3/4 cm - 1 1/4 cm). Transfer the peaches to a medium or large bowl. Add the blueberries, coconut sugar, cinnamon, nutmeg, ginger, and cardamom (if using), and toss gently to combine. Set aside for 30 minutes to allow the fruit to absorb the flavors.\nBrown the butter. Heat a large cast iron skillet (11- or 12-inch) over medium heat. Add the vegan butter to the pan, and use a spatula to spread it across the sides of the pan. Once it’s melted, foamy, and at a bubble (it should take 2 to 3 minutes), heat for another 2 minutes, stirring frequently to prevent burning, then take the pan off the heat.See note below on alternative pan sizes.**\nMake the cake. In a medium or large bowl, combine the flour, oats, cane sugar, salt, and baking powder. Whisk in the oat milk and vanilla extract and almond extract (if using) until well combined. Using a ladle or measuring cup, ladle the batter on top of the brown butter in the pan. Ladling, instead of pouring all of the batter on top at once, helps the butter swirl and mix into batter. Top the cobbler with the peach-blueberry mixture.\nBake the cobbler for 45-50 minutes, rotating the pan 180° halfway through to ensure even browning, until the top is deeply golden brown and bubbling.\nTransfer to a wire rack to cool for 10 minutes, then serve warm. If desired, scoop some vegan vanilla ice cream on each slice before serving.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.95, self.harvester_class.ratings())



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import normalize_string


class TimesOfIndia(AbstractScraper):
    @classmethod
    def host(cls):
        return "recipes.timesofindia.com"

    def author(self):
        return self.schema.author()

    def title(self):
        return self.schema.title()

    def total_time(self):
        return self.schema.total_time()

    def yields(self):
        return self.schema.yields()

    def image(self):
        return self.schema.image()

    def ingredients(self):
        ingredients = self.soup.find_all("label", attrs={"class": "clearfix"})

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        return self.schema.instructions()

    def ratings(self):
        return self.schema.ratings()

    def language(self):
        meta_language = self.soup.find("meta", attrs={"http-equiv": "content-language"})

        return normalize_string(meta_language.get("content"))

    def cuisine(self):
        return self.schema.cuisine()



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, normalize_string


class TheVintageMixer(AbstractScraper):
    @classmethod
    def host(cls):
        return "thevintagemixer.com"

    def title(self):
        return self.soup.find("h2", {"class": "wprm-recipe-name"}).get_text()

    def total_time(self):
        return get_minutes(
            self.soup.find("span", {"class": "wprm-recipe-total_time-minutes"}).parent
        )

    def image(self):
        container = self.soup.find("div", {"class": "wprm-recipe-image"})
        if not container:
            return None

        image = container.find("img", {"src": True})
        return image["src"] if image else None

    def ingredients(self):
        ingredients = self.soup.findAll("li", {"class": "wprm-recipe-ingredient"})

        return [
            normalize_string(ingredient.get_text())
            for ingredient in ingredients
            if len(normalize_string(ingredient.get_text())) > 0
        ]

    def instructions(self):
        instructions = self.soup.findAll(
            "div", {"class": "wprm-recipe-instruction-text"}
        )

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )



# mypy: disallow_untyped_defs=False
import re

from ._abstract import AbstractScraper
from ._utils import normalize_string

BULLET_CHARACTER_ORD = 8226


class NutritionByNathalie(AbstractScraper):
    @classmethod
    def host(cls):
        return "nutritionbynathalie.com"

    def title(self):
        return self.soup.find("h1").get_text()

    def image(self):
        try:
            return self.soup.find("img", {"id": re.compile(r"^innercomp_")})["src"]
        except Exception:
            return None

    def ingredients(self):
        ingredients = []
        element = self.soup.find(string=re.compile(r"Ingredients:"))
        parent_div = element.find_parent("div")
        paragraphs = parent_div.find_all("p")
        for paragraph in paragraphs:
            ingredient = paragraph.get_text()
            if ord(ingredient[0]) == BULLET_CHARACTER_ORD:
                ingredients.append(ingredient[2:])

        return ingredients

    def instructions(self):
        element = self.soup.find(string=re.compile("Directions:"))
        parent_div = element.find_parent("div")
        li_items = parent_div.find_all("li")

        return "\n".join([normalize_string(li_item.get_text()) for li_item in li_items])

    def ratings(self):
        return None



# mypy: disallow_untyped_defs=False
from ._abstract import AbstractScraper
from ._utils import get_minutes, get_yields, normalize_string


class MonsieurCuisine(AbstractScraper):
    @classmethod
    def host(cls):
        return "monsieur-cuisine.com"

    def author(self):
        container = self.soup.find("p", {"class": "recipe_author"})

        return container.span.get_text().strip()

    def title(self):
        return self.soup.h1.get_text()

    def total_time(self):
        container = self.soup.find("div", {"class": "duration-information"}).find(
            "div", {"class": "recipe-duration-total"}
        )

        total_time = container.b.get_text()
        total_time_hours = total_time.split(":")[0]
        total_time_mins = total_time.split(":")[1]

        return get_minutes(f"{total_time_hours}h and {total_time_mins}mins")

    def yields(self):
        container = self.soup.find("div", {"class": "recipe--info"}).find(
            "div", {"class": "recipe-portions"}
        )

        return get_yields(container.get_text())

    def image(self):
        container = self.soup.find("div", {"class", "flexed-image-preview"})
        container = container.find("figure")
        container = container and container.find("img")
        if container and container.has_attr("src"):
            container = container["src"]

        if not container:
            return None

        return "https://www." + self.host() + container

    def ingredients(self):
        container = self.soup.find(
            "div", {"class": "recipe--ingredients-html-item col-md-8"}
        )

        ingredients = container.findAll("li")

        return [normalize_string(ingredient.get_text()) for ingredient in ingredients]

    def instructions(self):
        container = self.soup.find("div", {"class": "recipe--instructions"})

        instructions = container.find_all("li")

        return "\n".join(
            [normalize_string(instruction.get_text()) for instruction in instructions]
        )

    def ratings(self):
        container = self.soup.find("div", {"class": "recipe--rating"})

        full_stars = container.find_all("span", {"class": "mc-star_filled"})
        half_stars = container.find_all("span", {"class": "mc-half_star_thick"})
        return len(full_stars) + 0.5 * len(half_stars)



from recipe_scrapers.joyfoodsunshine import Joyfoodsunshine
from tests import ScraperTest


class TestJoyfoodsunshineScraper(ScraperTest):

    scraper_class = Joyfoodsunshine

    def test_host(self):
        self.assertEqual("joyfoodsunshine.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "https://joyfoodsunshine.com/easy-homemade-pizza-dough/",
            self.harvester_class.canonical_url(),
        )

    def test_author(self):
        self.assertEqual("Laura", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Easy Homemade Pizza Dough", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(30, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("1 serving", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://joyfoodsunshine.com/wp-content/uploads/2016/06/easy-homemade-pizza-dough-recipe-4.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "▢ 1 cup warm water 105-110 degrees F",
                "▢ 1 TBS sugar",
                "▢ 1 TBS active dry yeast",
                "▢ 1 TBS olive oil",
                "▢ 2 to 2 ½ cups all-purpose flour*",
                "▢ 1 tsp salt",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Preheat oven to 450 degrees F. Grease a pizza pan or large jelly roll pan and set aside.\nPut warm water into a large mixing bowl.\nAdd sugar and yeast and stir to combine.\nLet mixture sit for 5 minutes, or until it becomes frothy and bubbles form.\nAdd olive oil and gently stir to combine.\nAdd 2 cups of flour and salt and mix with a spatula until a ball begins to form (dough will still be slightly sticky). Add more flour as needed to form a dough ball.\nTransfer to a floured surface and knead into a smooth dough, adding up to ½ cup extra flour if needed.\nRoll dough into your desired shape and gently transfer to your prepared pan.\nTo ensure the dough doesn't form air pockets, use a fork to prick to the dough all around (gently so you do not poke holes all the way through the crust).\nBake on the lower rack of your preheated oven for 5 minutes and remove from your oven. (this is just to very slightly prebake the dough). If you notice air bubbles forming, poke them with a fork to let the air out.\nAdd pizza sauce and toppings of choice!\nBake on the lower rack of your oven for around 15-20 minutes until the crust looks crispy and lightly browned.\nLet cool, cut and serve.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.96, self.harvester_class.ratings())



# mypy: allow-untyped-defs

from recipe_scrapers.chefnini import Chefnini
from tests import ScraperTest


class TestChefniniScraper(ScraperTest):
    scraper_class = Chefnini

    def test_host(self):
        self.assertEqual("chefnini.com", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("chefNini", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Tagliatelles carbonara", self.harvester_class.title())

    def test_category(self):
        self.assertEqual(None, self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual(None, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://static.chefnini.com/wp-content/uploads/2013/09/tagliatelle-carbonara10.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "250 g de tagliatelles",
                "100 g de lardons fumés ou de pancetta",
                "1 oeuf",
                "1 jaune d’oeuf",
                "4 cs de parmesan fraîchement râpé",
                "1 gousse d’ail",
                "1 peu de noix de muscade (facultatif)",
                "poivre du moulin",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "1- Versez l’œuf et le jaune dans un bol et battez-les à la fourchette.\n2- Ajoutez le parmesan et mélangez.\n3- Épluchez la gousse d’ail, dégermez-la et hachez-la.\n4- Portez une grande casserole d’eau salée à ébullition et mettez à cuire les pâtes en comptant 2 minutes de moins que le temps du paquet. Une fois cuites, égouttez-les mais gardez 2 louches d’eau de cuisson.\n5- Pendant ce temps, faites dorer les lardons dans une poêle, sans matière grasse.\n6- Ajoutez la gousse d’ail et laissez cuire en mélangeant régulièrement.\n7- Lorsque les pâtes sont cuites, versez-les dans les lardons et mélangez.\n8- Versez la préparation œuf / parmesan et une petite louche d’eau de cuisson. Mettez sur feu moyen et mélangez délicatement.\n9- Stoppez la cuisson lorsque la sauce est encore sirupeuse – trop tard, les pâtes seront sèches. Dosez l’eau de cuisson pour obtenir la consistance de sauce désirée.\n10- Servez aussitôt, poivrez et ajoutez un peu de noix de muscade râpée.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(None, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual(None, self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "En France, les pâtes carbonara sont très souvent cuisinées avec des oignons et de la crème liquide. C’est d’ailleurs de cette manière que je les ai découvertes. J’aimais bien mais j’ai toujours trouvé ça un peu écœurant et lourd.",
            self.harvester_class.description(),
        )



from recipe_scrapers.hellofresh import HelloFresh
from tests import ScraperTest


class TestHelloFreshScraperAdHoc(ScraperTest):

    scraper_class = HelloFresh
    test_file_name = "hellofresh_adhoc"

    def test_canonical_url(self):
        self.assertEqual(
            "https://www.hellofresh.fr/recipes/chakchouka-au-fromage-de-chevre-5eb41aa7d5418a482a67b864",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(
            "Chakchouka au fromage de chèvre avec du piment, des œufs et du persil",
            self.harvester_class.title(),
        )

    def test_author(self):
        self.assertEqual(self.harvester_class.author(), "HelloFresh")

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("2 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "2 * ¼ pièce Cube de bouillon de légumes",
                "2 * ½ pièce Oignon jaune",
                "2 * 1 pièce Gousse d'ail",
                "2 * ¼ pièce Piment rouge",
                "2 * ½ pièce Poivron rouge",
                "2 * 2 pièce Tomate allongée",
                "2 * 2 pièce Œuf de poule élevée en plein air",
                "2 * 50 g Fromage de chèvre frais",
                "2 * 2.5 g Persil plat",
                "2 * 1 pièce Demi-baguette",
                "2 * 1 cs Huile d'olive",
                "2 * selon le goût Poivre et sel",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Préchauffez le four à 210 degrés. Hachez l'oignon et écrasez ou émincez l'ail. Épépinez le piment rouge et le poivron rouge et coupez-les en lanières. Coupez les tomates prunes en petits dés et ciselez finement le persil frisé.\nFaites chauffer l’huile d’olive dans un wok ou une sauteuse avec couvercle et faites-y revenir l’oignon, l’ail et le piment rouge 2 minutes à feu moyen-vif. Puis, ajoutez le poivron, la tomate et la moitié du persil.\nAjoutez ensuite 30 ml d’eau par personne dans le wok ou la sauteuse, puis émiettez le cube de bouillon par-dessus. Portez à ébullition tout en remuant, puis laissez mijoter 5 minutes à couvert sur feu moyen-vif. Salez et poivrez.\nPour chaque œuf, faites un petit cratère dans le mélange de légumes, puis cassez-y les œufs. Saupoudrez-les de sel et de poivre. Disposez le chèvre frais émietté sur l’ensemble, couvrez et laissez les œufs cuire pendant 10 minutes. Lors des 4 dernières minutes, retirez le couvercle, augmentez un peu le feu et laissez l’excédent d’eau s’évaporer.\nPendant ce temps, enfournez les demi-baguettes 6 à 8 minutes ou jusqu’à ce qu’elles soient dorées.\nServez le chakchouka dans les assiettes. Garnissez du reste de persil et présentez les demi-baguettes en accompagnement.""",
            self.harvester_class.instructions(),
        )

    def test_nutrients(self):
        self.assertEqual(
            {
                "calories": "857 kcal",
                "fatContent": "28 g",
                "saturatedFatContent": "10 g",
                "carbohydrateContent": "109 g",
                "sugarContent": "15 g",
                "proteinContent": "35 g",
                "sodiumContent": "4 g",
                "servingSize": "654",
            },
            self.harvester_class.nutrients(),
        )

    def test_cuisine(self):
        self.assertEqual("Middle Eastern", self.harvester_class.cuisine())

    def test_category(self):
        self.assertEqual("main course", self.harvester_class.category())



from recipe_scrapers.nihhealthyeating import IngredientGroup, NIHHealthyEating
from tests import ScraperTest

# test recipe's URL
# https://healthyeating.nhlbi.nih.gov/recipedetail.aspx?cId=0&rId=188


class TestNIHHealthyEatingRecipesScraper(ScraperTest):

    scraper_class = NIHHealthyEating
    test_file_name = "nihhealthyeating_1"

    def test_host(self):
        self.assertEqual("healthyeating.nhlbi.nih.gov", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Baked Tilapia With Tomatoes")

    def test_description(self):
        self.assertEqual(
            "This dish is easy to prepare, low in calories, and economical.",
            self.harvester_class.description(),
        )

    def test_recipe_source(self):
        self.assertEqual(
            "Delicious Heart Healthy Latino Recipes",
            self.harvester_class.recipe_source(),
        )

    def test_total_time(self):
        self.assertEqual(35, self.harvester_class.total_time())

    def test_prep_time(self):
        self.assertEqual(10, self.harvester_class.prep_time())

    def test_cook_time(self):
        self.assertEqual(25, self.harvester_class.cook_time())

    def test_serving_size(self):
        self.assertEqual("1 fillet", self.harvester_class.serving_size())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_nutrients(self):
        self.assertEqual(
            {
                "calories": "265",
                "Total fat": "16 g",
                "Saturated fat": "2 g",
                "Cholesterol": "58 mg",
                "Sodium": "172 mg",
                "Total fiber": "3 g",
                "Protein": "22 g",
                "Carbohydrates": "9 g",
                "Potassium": "635 mg",
            },
            self.harvester_class.nutrients(),
        )

    def test_image(self):
        self.assertEqual(
            "https://healthyeating.nhlbi.nih.gov/images/food/Baked_Tilapia_With_Tomatoes.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "Cooking spray",
                "4 tilapia fillets",
                "4 medium tomatoes, peeled and chopped",
                "2 Tbsp olive oil",
                "1½ tsp thyme",
                "¼ C pitted black olives, diced",
                "¼ tsp red pepper flakes",
                "2 cloves garlic, minced",
                "½ C red onion, diced",
                "1 Tbsp lime juice",
                "Parsley and lemon wedges for garnish",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Preheat oven to 400 °F.\nSpray baking dish with cooking spray.\nArrange fillets in the baking dish. The dish should be large enough so the fillets do not overlap.\nMix remaining ingredients, except parsley and lemon wedges, in a bowl.\nSpoon the tomato mixture evenly over the fillets.\nBake uncovered 15 to 20 minutes or until the fish flakes easily with a fork.\nGarnish with parsley and a lemon wedge, and serve.",
            self.harvester_class.instructions(),
        )

    def test_recipe_cards(self):
        self.assertEqual(
            [
                {
                    "size": '4" x 6" (353 KB)',
                    "url": "http://www.nhlbi.nih.gov/health/public/heart/obesity/wecan/downloads/baked-tilapia-4x6.pdf",
                },
                {
                    "size": '8.5" x 11" (403 KB)',
                    "url": "http://www.nhlbi.nih.gov/health/public/heart/obesity/wecan/downloads/baked-tilapia-letter.pdf",
                },
            ],
            self.harvester_class.recipe_cards(),
        )


# test recipe's URL
# https://healthyeating.nhlbi.nih.gov/recipedetail.aspx?linkId=11&cId=1&rId=5


class TestNIHHealthyEatingRecipesVariationScraper(ScraperTest):

    scraper_class = NIHHealthyEating
    test_file_name = "nihhealthyeating_2"

    def test_host(self):
        self.assertEqual("healthyeating.nhlbi.nih.gov", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(
            self.harvester_class.title(),
            "Broiled Sirloin With Spicy Mustard and Apple Chutney",
        )

    def test_ingredient_groups(self):
        self.assertEqual(
            [
                IngredientGroup(
                    ingredients=[
                        "1 Granny Smith apple, rinsed, peeled, cored, and diced (about 1 C)",
                        "2 Tbsp shallots, minced",
                        "1 Tbsp garlic, minced (about 2–3 cloves)",
                        "½ C canned no-salt-added diced tomatoes",
                        "2 oz golden seedless raisins (about ½ C)",
                        "¼ C apple cider vinegar",
                        "2 Tbsp maple syrup",
                    ],
                    purpose="For chutney:",
                ),
                IngredientGroup(
                    ingredients=[
                        "4 beef top sirloin steaks, lean\xa0(3 oz each)",
                        "¼ tsp salt",
                        "¼ tsp ground black pepper",
                        "1 Tbsp olive oil",
                    ],
                    purpose="For steak:",
                ),
                IngredientGroup(
                    ingredients=[
                        "2 C low-sodium beef broth",
                        "2 Tbsp Dijon mustard",
                        "2 Tbsp cornstarch",
                    ],
                    purpose="For mustard dressing:",
                ),
            ],
            self.harvester_class.ingredient_groups(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 Granny Smith apple, rinsed, peeled, cored, and diced (about 1 C)",
                "2 Tbsp shallots, minced",
                "1 Tbsp garlic, minced (about 2–3 cloves)",
                "½ C canned no-salt-added diced tomatoes",
                "2 oz golden seedless raisins (about ½ C)",
                "¼ C apple cider vinegar",
                "2 Tbsp maple syrup",
                "4 beef top sirloin steaks, lean\xa0(3 oz each)",
                "¼ tsp salt",
                "¼ tsp ground black pepper",
                "1 Tbsp olive oil",
                "2 C low-sodium beef broth",
                "2 Tbsp Dijon mustard",
                "2 Tbsp cornstarch",
            ],
            self.harvester_class.ingredients(),
        )
        # check length of ingredients() === sum of lengths of ingredient_groups()
        self.assertEqual(
            len(self.harvester_class.ingredients()),
            sum(
                [len(ig.ingredients) for ig in self.harvester_class.ingredient_groups()]
            ),
        )

    def test_recipe_cards(self):
        self.assertEqual(None, self.harvester_class.recipe_cards())


# test recipe's URL
# https://healthyeating.nhlbi.nih.gov/recipedetail.aspx?linkId=0&cId=10&rId=163


class TestNIHHealthyEatingRecipesEdgeCaseScraper(ScraperTest):

    scraper_class = NIHHealthyEating
    test_file_name = "nihhealthyeating_3"

    def test_host(self):
        self.assertEqual("healthyeating.nhlbi.nih.gov", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Fruit Skewers With Yogurt Dip")

    def test_recipe_source(self):
        self.assertEqual(
            "Deliciously Healthy Family Meals",
            self.harvester_class.recipe_source(),
        )

    def test_ingredient_groups(self):
        self.assertEqual(
            [
                IngredientGroup(
                    ingredients=[
                        "1 C strawberries, rinsed, stems removed, and cut in half",
                        "¼ C fat-free plain yogurt",
                        "1/8 tsp vanilla extract",
                        "1 Tbsp honey",
                    ],
                    purpose="For dip:",
                ),
                IngredientGroup(
                    ingredients=[
                        "1 C strawberries, rinsed, stems removed, and cut in half",
                        "1 C fresh pineapple, diced (or canned pineapple chunks in juice, drained)",
                        "½ C blackberries",
                        "1 tangerine or Clementine, peeled and cut into 8 segments",
                        "8 6-inch wooden skewers",
                    ],
                    purpose=None,
                ),
            ],
            self.harvester_class.ingredient_groups(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "1 C strawberries, rinsed, stems removed, and cut in half",
                "¼ C fat-free plain yogurt",
                "1/8 tsp vanilla extract",
                "1 Tbsp honey",
                "1 C strawberries, rinsed, stems removed, and cut in half",
                "1 C fresh pineapple, diced (or canned pineapple chunks in juice, drained)",
                "½ C blackberries",
                "1 tangerine or Clementine, peeled and cut into 8 segments",
                "8 6-inch wooden skewers",
            ],
            self.harvester_class.ingredients(),
        )
        # check length of ingredients() === sum of lengths of ingredient_groups()
        self.assertEqual(
            len(self.harvester_class.ingredients()),
            sum(
                [len(ig.ingredients) for ig in self.harvester_class.ingredient_groups()]
            ),
        )



from recipe_scrapers.usdamyplate import USDAMyPlate
from tests import ScraperTest

# test recipe's URL
# https://www.myplate.gov/recipes/supplemental-nutrition-assistance-program-snap/fabulous-fig-bars


class TestUSDAMyPlateRecipesScraper(ScraperTest):

    scraper_class = USDAMyPlate

    def test_host(self):
        self.assertEqual("myplate.gov", self.harvester_class.host())

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Fabulous Fig Bars")

    def test_total_time(self):
        self.assertEqual(75, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("24 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "3 cups dried figs (chopped, 16 ounces)",
                "1/2 cup walnuts (chopped)",
                "1/3 cup sugar",
                "1/4 cup orange juice (juice from 1/2 orange)",
                "2 tablespoons hot water",
                "1/2 cup margarine (softened, or butter)",
                "1 cup packed brown sugar",
                "1 large egg",
                "1 1/2 cups all-purpose flour",
                "1/2 teaspoon baking soda",
                "1 1/4 cups old fashioned rolled oats",
            ],
            self.harvester_class.ingredients(),
        )

    def test_image(self):
        self.assertEqual(
            "https://myplate-prod.azureedge.net/sites/default/files/styles/recipe_525_x_350_/public/2021-03/FigBars.jpg",
            self.harvester_class.image(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "Wash hands with soap and water.\nPreheat oven to 350 degrees F. Lightly grease a 9x13-inch baking pan.\nCombine figs, walnuts, sugar, orange juice and hot water in a mixing bowl and set aside.\nMix together margarine or butter and brown sugar until creamy. Add egg and mix until smooth.\nMix flour and baking soda. Stir into egg mixture. Blend in oats to make soft dough.\nReserve 1 cup of dough for topping. With floured fingertips, press the remaining dough into a thin layer on the bottom of the baking pan.\nSpread fig mixture evenly over the dough. Crumble reserved dough over top, allowing fig mixture to show.\nBake 30 minutes or until golden brown. Cool completely in baking pan. Cut into 24 bars (about 2.5 x 2 inches).",
            self.harvester_class.instructions(),
        )

    def test_nutrients(self):
        self.assertEqual(
            {
                "Total Calories": "206",
                "Total Fat": "6 g",
                "Saturated Fat": "1 g",
                "Cholesterol": "8 mg",
                "Sodium": "66 mg",
                "Dietary Fiber": "3 g",
                "Protein": "3 g",
                "Carbohydrates": "37 g",
                "Potassium": "225 mg",
                "Calcium": "55 mg",
                "Vitamin D": "0 mcg",
                "Iron": "1 mg",
                "Total Sugars": "24 g",
                "Added Sugars included": "11 g",
            },
            self.harvester_class.nutrients(),
        )

    def test_serving_size(self):
        self.assertEqual("1 bar", self.harvester_class.serving_size())

    def test_description(self):
        self.assertEqual(
            "Fig bars are a great on-the-go snack. The sweet, nutty flavors in these bars are sure to be a hit with all ages.",
            self.harvester_class.description(),
        )

    def test_recipe_source(self):
        self.assertEqual(
            "Fabulous Fig Bars from Food Hero, Oregon State University, Cooperative Extension Service.",
            self.harvester_class.recipe_source(),
        )



# mypy: allow-untyped-defs

from recipe_scrapers.projectgezond import ProjectGezond
from tests import ScraperTest


class TestProjectGezondScraper(ScraperTest):

    scraper_class = ProjectGezond

    def test_host(self):
        self.assertEqual("projectgezond.nl", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Project Gezond", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Boeuf bourguignon", self.harvester_class.title())

    def test_category(self):
        self.assertEqual("Diner, Kerstrecepten", self.harvester_class.category())

    def test_total_time(self):
        self.assertEqual("30 minuten + 2 uur stoven", self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("twee personen", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://www.projectgezond.nl/wp-content/uploads/2021/11/BoeufBourguignon-768x1024.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "40 gr ontbijtspek",
                "250 gr runderriblappen",
                "10 gr bloem",
                "1 ui",
                "150 gr winterpeen",
                "1 teentje knoflook",
                "35 gr tomatenpuree",
                "100 ml rode wijn",
                "200 ml runderbouillon",
                "1 laurierblaadje",
                "1 takje tijm",
                "1 kruidnagel",
                "150 gr champignons",
                "50 gr zilveruitjes",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Bak de plakken ontbijtspek bruin en licht krokant in een droge (stoof)pan. Haal uit de pan en zorg dat het bakvet achterblijft.\nSnijd de runderriblappen in blokjes van 2 bij 2 centimeter. Bestrooi met peper, zout en de bloem. Schep om tot alles goed verdeeld is. \nSnijd de ui in halve ringen en de winterpeen in plakken.\nHak de knoflook fijn. \nVerwarm de pan waar het ontbijtspek in gebakken is opnieuw. Bak de riblappen op hoog vuur rondom bruin. \nGebruik indien nodig een klein beetje boter of olijfolie. \nVoeg de ui en winterpeen toe en bak enkele minuten mee met de blokjes vlees.\nZet het vuur lager en voeg de knoflook toe. Bak 1 à 2 minuten mee. Voeg de tomatenpuree toe. Roer los en bak 2 à 3 minuten mee.\nBlus af met de rode wijn. Roer eventuele aanbaksels los van de bodem. Laat de wijn grotendeels verdampen.\nVoeg de runderbouillon, het laurierblaadje, het takje tijm en de kruidnagel toe.\nSnijd de plakken ontbijtspek in stukjes en voeg toe.\nLaat het gerecht ongeveer 2 uur stoven met de deksel op de pan. \nBoen de champignons schoon en snijd ze in kwarten. \nSnijd de zilveruitjes doormidden. \nVoeg de champignons en de zilveruitjes toe.\nLaat alles nog minimaal 30 minuten stoven. Doe dit eventueel zonder deksel op de pan, zodat de boeuf bourguignon wat verder inkookt. \nHaal het laurierblaadje, het takje tijm en de kruidnagel uit de pan.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(None, self.harvester_class.ratings())

    def test_cuisine(self):
        self.assertEqual(None, self.harvester_class.cuisine())

    def test_description(self):
        self.assertEqual(
            "Deze klassieker vindt zijn oorsprong in de Franse keuken. ‘Rund op bourgondische wijze’ is een vertaling van ‘Boeuf bourguignon’ die niets aan de verbeelding overlaat.\nDit recept is dan ook het perfecte antwoord als het weer tijd is voor een potje stoof!\nWant het is verre van moeilijk om deze ultieme stoofpot te bereiden. Je hebt enkel wat (wacht)tijd en dus geduld nodig. Het is helemaal geen gek idee dat je dit recept al de dag van tevoren klaarmaakt trouwens. De smaken kunnen dan zelfs nog beter intrekken.\nAls dat geen ‘Boeuf bourguignon’ wordt…",
            self.harvester_class.description(),
        )



# mypy: allow-untyped-defs

from recipe_scrapers.rutgerbakt import RutgerBakt
from tests import ScraperTest


class TestRutgerBaktScraper(ScraperTest):
    scraper_class = RutgerBakt
    test_file_name = "rutgerbakt_3"

    def test_host(self):
        self.assertEqual("rutgerbakt.nl", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Rutger van den Broek", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Banketstaaf", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(45, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual(None, self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://rutgerbakt.nl/wp-content/uploads/2016/11/banketstaaf-1-e1478017268526-1920x1080-c-default.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "400 gr amandelspijs, gekocht of zelfgemaakt",
                "1 rol Tante Fanny vers bladerdeeg 270 gr",
                "½ ei, losgeklopt",
                "½ ei, losgeklopt, om te bestrijken",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Verwarm de oven voor op 220 ⁰C en bekleed een bakplaat met bakpapier. Kneed het amandelspijs kort door en voeg vervolgens een half ei toe om het amandelspijs iets soepeler te maken. Maak twee rollen van het amandelspijs van zo’n 32-34 centimeter lang.\nRol het bladerdeeg uit en snijd het deeg in de lengte doormidden. Leg op het midden van ieder stuk bladerdeeg een rol amandelspijs en vouw de twee korte uiteinden van het deeg over het amandelspijs. Bestrijk deze twee uiteinden licht met water. Vouw één lange kant van het deeg over het amandelspijs en maak ook deze licht vochtig. Vouw tot slot de andere kant van het deeg over de rol amandelspijs en druk dit goed vast. Leg de banketstaven met de deegnaad naar beneden op de bakplaat en laat ze 20 minuten rusten in de koelkast.\nBestrijk de banketstaven met het losgeklopte ei en bak ze in 25 tot 35 minuten goudbruin. Laat de banketstaven afkoelen op een rooster.\n\n\nDit bericht is gesponsord door Tante Fanny.\n",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.0, self.harvester_class.ratings())

    def test_description(self):
        self.assertEqual(
            "Rondom Sinterklaas wordt er volop gebakken voor het ‘heerlijk avondje’! Een lekkernij die zeker niet mag ontbreken is de banketstaaf. Als kind (en nu eigenlijk nog steeds wel) wilde ik altijd het liefst het kontje van de banketstaaf. Er is niks lekkerder dan eerst het spijs eruit te lepelen en dan als laatste het bladerdeeg op te eten. Heerlijk!",
            self.harvester_class.description(),
        )



# mypy: allow-untyped-defs

from recipe_scrapers.rutgerbakt import RutgerBakt
from tests import ScraperTest


class TestRutgerBaktScraper(ScraperTest):
    scraper_class = RutgerBakt
    test_file_name = "rutgerbakt_1"

    def test_host(self):
        self.assertEqual("rutgerbakt.nl", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("Rutger van den Broek", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Arretjescake", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(180, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual(None, self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://rutgerbakt.nl/wp-content/uploads/2020/10/arretjescake-scaled-1920x1080-c-default.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "75 ml slagroom",
                "2 eidooiers",
                "50 +150 gr witte basterdsuiker",
                "50 gr cacaopoeder",
                "225 gr boter, gesmolten",
                "225 gr biscuits (Maria biscuits)",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            "Begin met het maken van de custard. Doe de slagroom in een steelpannetje en breng dit aan de kook. Doe de eidooiers met de 50 gram basterdsuiker in een kom en klop deze door elkaar. Schenk de kokende slagroom al roerende op het eidooiermengsel en klop alles goed door. Giet het mengsel terug in de pan. Verwarm het op laag vuur en blijf roeren tot het mengsel dikker begint te worden. De custard mag niet koken, verwarm deze tot ongeveer 85 °C. Haal de pan dan van het vuur.\nDoe de rest van de basterdsuiker met het cacaopoeder en de custard in een kom en klop dit met een (hand)mixer met garde(s) door elkaar. Voeg de gesmolten boter toe en klop alles door tot de suiker opgelost is. Dit kun je testen door wat van het mengsel tussen duim en wijsvinger te pakken. Voel je nog korrels van de suiker, dan moet je nog wat langer kloppen.\nDoe de biscuitjes in een schone theedoek en vouw deze dicht. Sla met een deegroller (of een ander hard voorwerp) op de doek zodat de koekjes in stukjes breken. Voeg de biscuitstukjes toe aan het mengsel en roer door elkaar. Bekleed de binnenkant van een cakevorm van 25 centimeter met plasticfolie en laat het folie aan de bovenkant overhangen. Schep het mengsel in de vorm, druk het goed aan en dek het af met het overhangende plasticfolie. Laat de arretjescake 2-3 uur opstijven in de koelkast. Haal de cake met behulp van het folie uit de vorm en snijd ‘m in plakken.",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(4.0, self.harvester_class.ratings())

    def test_description(self):
        self.assertEqual(
            "Maak zelf de lekkerste arretjescake met dit recept! Serveer plakjes of kleine stukjes arretjescake tijdens de koffie of als onderdeel van een high tea. Deze traktatie van cacao, koekjes en boter is helemaal niet moeilijk om te maken en perfect als je op zoek bent naar iets lekkers om van te voren te maken of een no-bake lekkernij. Voor deze arretjescake heb je namelijk geen oven nodig, maar je laat hem opstijven in de koelkast.\xa0",
            self.harvester_class.description(),
        )



from recipe_scrapers.foodrepublic import FoodRepublic
from tests import ScraperTest


class TestFoodRepublicScraper(ScraperTest):

    scraper_class = FoodRepublic

    def test_host(self):
        self.assertEqual("foodrepublic.com", self.harvester_class.host())

    def test_canonical_url(self):
        self.assertEqual(
            "http://www.foodrepublic.com/recipes/dutch-white-asparagus-recipe/",
            self.harvester_class.canonical_url(),
        )

    def test_title(self):
        self.assertEqual(self.harvester_class.title(), "Dutch White Asparagus Recipe")

    def test_total_time(self):
        self.assertEqual(60, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("4 servings", self.harvester_class.yields())

    def test_ingredients(self):
        self.assertEqual(
            [
                "12 spears Dutch white asparagus",
                "1/2 each Meyer lemon, orange and grapefruit",
                "2 cups water, for steaming",
                "1 cup grated Parmesan cheese",
                "1/2 cup Chardonnay",
                "1/2 cup white wine vinegar",
                "2 sprigs fresh thyme",
                "1/2 cup shallots, minced",
                "small chunk of Parmesan rind",
                "4 tablespoons unsalted butter, chilled and cut into small cubes",
                "4 fresh eggs, poached (see instructions)",
                "4 slices prosciutto",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        return self.assertEqual(
            "After peeling and trimming the asparagus, steam it in citrus water — water with Meyer lemon, orange and grapefruit slices — until fork tender.\nIn a pot add the Chardonnay, white wine vinegar, thyme, shallots and chunk of Parmesan rind; reduce about a quarter.\nRemove from heat and slowly whisk in cubes of butter until sauce is thick and glossy.\nPour 2 tablespoons of beurre blanc on a plate, top it with a poached egg, steamed white asparagus, prosciutto and grated Parmesan.\nSpring Asparagus Soup Recipe\nGoat Cheese And Asparagus Macaroni Salad Recipe\nRoasted Asparagus & Scrambled Eggs Recipe",
            self.harvester_class.instructions(),
        )



from recipe_scrapers.alltomat import AllTomat
from tests import ScraperTest


class TestAllTomatScraper(ScraperTest):

    scraper_class = AllTomat

    def test_host(self):
        self.assertEqual("alltommat.se", self.harvester_class.host())

    def test_author(self):
        self.assertEqual("gunilla von heland", self.harvester_class.author())

    def test_title(self):
        self.assertEqual("Brioche-hamburgerbröd", self.harvester_class.title())

    def test_total_time(self):
        self.assertEqual(20, self.harvester_class.total_time())

    def test_yields(self):
        self.assertEqual("12 servings", self.harvester_class.yields())

    def test_image(self):
        self.assertEqual(
            "https://cached-images.bonnier.news/bnl01/standard-article/ec93ddbf-20de-46db-a563-3b88881f0e71/89ad5fc0-7218-4e14-b9be-dafb51ae221a/16x9/0/original.jpg",
            self.harvester_class.image(),
        )

    def test_ingredients(self):
        self.assertEqual(
            [
                "2.5 dl vatten",
                "0.5 dl mjölk",
                "25 g jäst",
                "2.5 msk strösocker",
                "1 tsk salt",
                "1 ägg",
                "7.5 dl vetemjöl",
                "50 g smör",
                "1 ägg",
            ],
            self.harvester_class.ingredients(),
        )

    def test_instructions(self):
        self.assertEqual(
            """Värm vatten och mjölk till 37°. Rör ut jästen med lite av vätskan och tillsätt socker, salt, ägg och mjöl, spara ca 1/2 dl till utbakning. Arbeta degen i maskin ca 5 min. Klicka ner smöret mot slutet. Låt jäsa övertäckt ca 1 timme.\nArbeta degen lätt i maskinen. Ta upp på mjölad arbetsbänk. Dela degen i 12 bitar. Forma bitarna till runda bullar och platta ut dem lätt. Lägg dem på en plåt med bakplåtspapper. Låt jäsa övertäckta ytterligare ca 1 timme. Sätt ugnen på 200°.\nPensla bröden med uppvispat ägg och grädda mitt i ugnen ca 15 min. Låt svalna på galler.""",
            self.harvester_class.instructions(),
        )

    def test_ratings(self):
        self.assertEqual(3.9, self.harvester_class.ratings())
